@@ -2,7 +2,7 @@
 *******************************************************************************
 **
 **  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
+**  Copyright (C) 2004-2006 Red Hat, Inc.  All rights reserved.
 **
 **  This copyrighted material is made available to anyone wishing to use,
 **  modify, copy, or redistribute it subject to the terms and conditions
@@ -145,7 +145,7 @@ stuck_releasepage(struct buffer_head *bh)
  */
 
 static int
-gfs_aspace_releasepage(struct page *page, int gfp_mask)
+gfs_aspace_releasepage(struct page *page, gfp_t gfp_mask)
 {
 	struct inode *aspace = page->mapping->host;
 	struct gfs_sbd *sdp = get_v2sdp(aspace->i_sb);
@@ -839,6 +839,8 @@ gfs_dunpin(struct gfs_sbd *sdp, struct buffer_head *bh, struct gfs_trans *tr)
  * @bh: the buffer
  * @uptodate: whether or not the write succeeded
  *
+ * Interrupt context, no ENTER/RETURN
+ *
  */
 
 static void
