@@ -2,7 +2,7 @@
 *******************************************************************************
 **
 **  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
+**  Copyright (C) 2004-2006 Red Hat, Inc.  All rights reserved.
 **
 **  This copyrighted material is made available to anyone wishing to use,
 **  modify, copy, or redistribute it subject to the terms and conditions
@@ -14,11 +14,11 @@
 #ifndef __GFS_DOT_H__
 #define __GFS_DOT_H__
 
-#define GFS_RELEASE_NAME "1.04.00"
+#define GFS_RELEASE_NAME "2.00.00"
 
-#include <linux/gfs_ondisk.h>
+#include <linux/lm_interface.h>
 
-#include "lm_interface.h"
+#include "gfs_ondisk.h"
 #include "fixed_div64.h"
 #include "lvb.h"
 #include "incore.h"
