@@ -2,7 +2,7 @@
 *******************************************************************************
 **
 **  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
+**  Copyright (C) 2004-2006 Red Hat, Inc.  All rights reserved.
 **
 **  This copyrighted material is made available to anyone wishing to use,
 **  modify, copy, or redistribute it subject to the terms and conditions
@@ -1159,10 +1159,15 @@ gfs_glock_xmote_th(struct gfs_glock *gl, unsigned int state, int flags)
 			      gl->gl_state, state,
 			      lck_flags);
 
+	if (gfs_assert_withdraw(sdp, !(lck_ret & LM_OUT_ERROR)))
+		goto out;
+
 	if (lck_ret & LM_OUT_ASYNC)
 		gfs_assert_warn(sdp, lck_ret == LM_OUT_ASYNC);
 	else
 		xmote_bh(gl, lck_ret);
+ out:
+	return;
 }
 
 /**
@@ -1250,10 +1255,15 @@ gfs_glock_drop_th(struct gfs_glock *gl)
 
 	ret = gfs_lm_unlock(sdp, gl->gl_lock, gl->gl_state);
 
+	if (gfs_assert_withdraw(sdp, !(ret & LM_OUT_ERROR)))
+		goto out;
+
 	if (!ret)
 		drop_bh(gl, ret);
 	else
 		gfs_assert_warn(sdp, ret == LM_OUT_ASYNC);
+ out:
+	return;
 }
 
 /**
@@ -2126,6 +2136,7 @@ gfs_lvb_unhold(struct gfs_glock *gl)
 	glock_put(gl);
 }
 
+#if 0
 /**
  * gfs_lvb_sync - sync a LVB
  * @gl: The glock in question
@@ -2145,6 +2156,7 @@ gfs_lvb_sync(struct gfs_glock *gl)
 
 	unlock_on_glock(gl);
 }
+#endif
 
 /**
  * blocking_cb -
@@ -2194,9 +2206,9 @@ blocking_cb(struct gfs_sbd *sdp, struct lm_lockname *name, unsigned int state)
  */
 
 void
-gfs_glock_cb(lm_fsdata_t *fsdata, unsigned int type, void *data)
+gfs_glock_cb(void *fsdata, unsigned int type, void *data)
 {
-	struct gfs_sbd *sdp = (struct gfs_sbd *)fsdata;
+	struct gfs_sbd *sdp = fsdata;
 
 	atomic_inc(&sdp->sd_lm_callbacks);
 
@@ -2229,7 +2241,7 @@ gfs_glock_cb(lm_fsdata_t *fsdata, unsigned int type, void *data)
 
 	case LM_CB_NEED_RECOVERY:
 		gfs_add_dirty_j(sdp, *(unsigned int *)data);
-		if (test_bit(SDF_RECOVERD_RUN, &sdp->sd_flags))
+		if (sdp->sd_recoverd_process)
 			wake_up_process(sdp->sd_recoverd_process);
 		return;
 
@@ -2697,7 +2709,7 @@ gfs_gl_hash_clear(struct gfs_sbd *sdp, int wait)
 		}
 
 		invalidate_inodes(sdp->sd_vfs);
-		yield();
+		schedule_timeout_interruptible(HZ / 10);
 	}
 }
 
