@@ -83,6 +83,12 @@ int check_entries(struct fsck_inode *ip, osi_buf_t *bh, int index,
 			  }*/
 		}
 
+		if (de.de_rec_len < sizeof(struct gfs_dirent)) {
+			log_err("Entry %"PRIu64" of directory %"
+				PRIu64" is corrupt, skipping.\n",
+				BH_BLKNO(bh), ip->i_di.di_num.no_addr);
+			break;
+		}
 		if ((char *)dent + de.de_rec_len >= bh_end){
 			log_debug("Last entry processed.\n");
 			break;
@@ -184,6 +190,14 @@ int check_leaf(struct fsck_inode *ip, int *update, struct metawalk_fxns *pass)
 			}
 			gfs_leaf_in(&leaf, BH_DATA(lbh));
 
+			/* Make sure it's really a leaf. */
+			if (leaf.lf_header.mh_type != GFS_METATYPE_LF) {
+				log_err("Inode %" PRIu64 " points to bad leaf "
+					PRIu64 ".\n", ip->i_di.di_num.no_addr,
+					leaf_no);
+				relse_buf(sbp, lbh);
+				break;
+			}
 			exp_count = (1 << (ip->i_di.di_depth - leaf.lf_depth));
 			log_debug("expected count %u - %u %u\n", exp_count,
 				  ip->i_di.di_depth, leaf.lf_depth);
@@ -258,6 +272,7 @@ static int check_eattr_entries(struct fsck_inode *ip, osi_buf_t *bh,
 	uint64_t *ea_data_ptr = NULL;
 	int i;
 	int error = 0;
+	uint32_t offset = (uint32_t)sizeof(struct gfs_meta_header);
 
 	if(!pass->check_eattr_entry) {
 		return 0;
@@ -298,11 +313,11 @@ static int check_eattr_entries(struct fsck_inode *ip, osi_buf_t *bh,
 				}
 			}
 		}
-		if(ea_hdr->ea_flags & GFS_EAFLAG_LAST){
-			/* FIXME: better equal the end of the block */
+		offset += gfs32_to_cpu(ea_hdr->ea_rec_len);
+		if(ea_hdr->ea_flags & GFS_EAFLAG_LAST ||
+		   offset >= ip->i_sbd->sb.sb_bsize || ea_hdr->ea_rec_len == 0){
 			break;
 		}
-		/* FIXME: be sure this doesn't go beyond the end */
 		ea_hdr_prev = ea_hdr;
 		ea_hdr = (struct gfs_ea_header *)
 			((char *)(ea_hdr) +
