@@ -24,12 +24,11 @@
 #include <stdint.h>
 #include <inttypes.h>
 
-#include <linux/gfs_ondisk.h>
+#include "gfs_ondisk.h"
 #define __user
-#include <linux/gfs_ioctl.h>
+#include "gfs_ioctl.h"
 #include "osi_list.h"
-typedef uint64_t uint64;
-#include "iddev.h"
+#include "libgfs.h"
 
 #include "copyright.cf"
 
@@ -669,7 +668,7 @@ make_rgrp(uint64_t offset, uint64_t size)
 	rgl->rg.rg_header.mh_format = GFS_FORMAT_RG;
 	rgl->rg.rg_free = rgl->ri.ri_data;
 
-	osi_list_add(&rgl->list, &rglist_new);
+	osi_list_add_prev(&rgl->list, &rglist_new);
 	return offset + size;
 }
 
