@@ -11,6 +11,7 @@ typedef struct __attribute__ ((packed)) {
     uint32_t gh_command;
     uint32_t gh_arg1;
     uint32_t gh_arg2;
+    uint32_t gh_arg3;
 } generic_msg_hdr;
 
 #define swab_generic_msg_hdr(ptr)\
@@ -28,9 +29,8 @@ typedef struct __attribute__ ((packed)) {
 	char 		d_svcName[64];
         uint32_t	d_action;
         uint32_t	d_svcState;
-        uint64_t	d_svcOwner;
+        uint32_t	d_svcOwner;
         int32_t		d_ret;
-	uint32_t	d_pad;
     } sm_data;
 } SmMessageSt;
 
@@ -39,7 +39,7 @@ typedef struct __attribute__ ((packed)) {
 	swab_generic_msg_hdr(&((ptr)->sm_hdr));\
 	swab32((ptr)->sm_data.d_action);\
 	swab32((ptr)->sm_data.d_svcState);\
-	swab64((ptr)->sm_data.d_svcOwner);\
+	swab32((ptr)->sm_data.d_svcOwner);\
 	swab32((ptr)->sm_data.d_ret);\
 }
 
@@ -55,13 +55,10 @@ typedef struct __attribute__ ((packed)) {
 }
 
 
-#define GENERIC_HDR_MAGIC 0x123abc00/* match librhcm!  */
+#define GENERIC_HDR_MAGIC   0x123abc00
+#define GENERIC_HDR_MAGICV2 0x123abc02
 
-#ifndef clu_perror
-#define clu_perror(func, val) fprintf(stderr, "%s: %s\n", func, strerror(val)
-#endif
-
-int msg_send_simple(int fd, int cmd, int arg1, int arg2);
-int msg_receive_simple(int fd, generic_msg_hdr ** buf, int timeout);
+int msg_send_simple(msgctx_t *ctx, int cmd, int arg1, int arg2);
+int msg_receive_simple(msgctx_t *ctx, generic_msg_hdr ** buf, int timeout);
 
 #endif
