@@ -53,6 +53,10 @@
 #define	RS_RECOVER	(7)
 #define RS_CONDSTART	(8)	/** Start if flagged with RF_NEEDSTART */
 #define RS_CONDSTOP	(9)	/** STOP if flagged with RF_NEEDSTOP */
+#define RS_MONITOR	(10)
+#define RS_META_DATA	(11)
+#define RS_VALIDATE	(12)
+#define RS_MIGRATE	(13)
 
 
 #ifndef SHAREDIR
@@ -71,18 +75,19 @@
 
 
 typedef struct _resource_attribute {
-	int	ra_flags;
-	/* XXX possible alignment problem on ia64 */
 	char	*ra_name;
 	char	*ra_value;
+	int	ra_flags;
+	int	_pad_;
 } resource_attr_t;
 
 
 typedef struct _resource_child {
-	char    rc_forbid;
+	char	*rc_name;
 	int	rc_startlevel;
 	int	rc_stoplevel;
-	char	*rc_name;
+	int	rc_forbid;
+	int	_pad_;
 } resource_child_t;
 
 
@@ -92,6 +97,7 @@ typedef struct _resource_act {
 	time_t	ra_last;
 	time_t	ra_interval;
 	int	ra_depth;
+	int	ra_flags;
 } resource_act_t;
 
 
@@ -100,7 +106,7 @@ typedef struct _resource_rule {
 	char *	rr_type;
 	char *	rr_agent;
 	char *	rr_version;	/** agent XML spec version; OCF-ism */
-	int	rr_root;
+	int	rr_flags;
 	int	rr_maxrefs;
 	resource_attr_t *	rr_attrs;
 	resource_child_t *	rr_childtypes;
@@ -135,7 +141,7 @@ typedef struct _fod_node {
 	list_head();
 	char	*fdn_name;
 	int	fdn_prio;
-	int	_pad_; /* align */
+	int	fdn_nodeid;
 } fod_node_t;
 
 typedef struct _fod {
@@ -192,7 +198,7 @@ void destroy_resource_tree(resource_node_t **tree);
 int construct_domains(int ccsfd, fod_t **domains);
 void deconstruct_domains(fod_t **domains);
 void print_domains(fod_t **domains);
-int node_should_start(uint64_t nodeid, cluster_member_list_t *membership,
+int node_should_start(int nodeid, cluster_member_list_t *membership,
 		      char *rg_name, fod_t **domains);
 
 
