@@ -20,7 +20,7 @@
 /** @file
  * Library routines for communicating with the logging daemon.
  *
- *  $Id: clulog.c,v 1.2.2.1.6.5 2007/01/26 20:47:16 lhh Exp $
+ *  $Id: clulog.c,v 1.5 2006/06/02 17:37:10 lhh Exp $
  *
  *  Author: Jeff Moyer <moyer@missioncriticallinux.com>
  */
@@ -50,7 +50,7 @@
 #include <string.h>
 
 
-static const char *version __attribute__ ((unused)) = "$Revision: 1.2.2.1.6.5 $";
+static const char *version __attribute__ ((unused)) = "$Revision: 1.5 $";
 
 #ifdef DEBUG
 #include <assert.h>
@@ -183,14 +183,6 @@ int
 clu_log_console(int onoff)
 {
 	int ret = useconsole;
-	int val;
-
-	if (onoff) {
-		val = fcntl(STDERR_FILENO, F_GETFL, 0);
-		fcntl(STDERR_FILENO, F_SETFL, val|O_NONBLOCK);
-		val = fcntl(STDOUT_FILENO, F_GETFL, 0);
-		fcntl(STDOUT_FILENO, F_SETFL, val|O_NONBLOCK);
-	}
 
 	useconsole = !!onoff;
 	return ret;
@@ -212,7 +204,6 @@ do_clulog(int        severity,
 	char         logmsg[MAX_LOGMSG_LEN];	/* message to go to the log */
 	char         printmsg[MAX_LOGMSG_LEN];	/* message to go to stdout */
 	int          syslog_flags = LOG_NDELAY;
-	int	     val;
 
 	pthread_mutex_lock(&log_mutex);
 	if (severity > loglevel) {
@@ -284,19 +275,9 @@ do_clulog(int        severity,
 			  MAX_LOGMSG_LEN - strlen(printmsg), fmt, args);
 		va_end(args);
 
-		if (useconsole && !write_to_cons) {
-			val = fcntl(STDOUT_FILENO, F_GETFL, 0);
-			fcntl(STDOUT_FILENO, F_SETFL, val | O_NONBLOCK);
-		}
-
-		/* Ignore error return code */
-		write(STDOUT_FILENO, printmsg, strlen(printmsg));
-
-		if (useconsole && !write_to_cons)
-			fcntl(STDOUT_FILENO, F_SETFL, val);
+		fprintf(stdout, "%s", printmsg);
 	}
 
-	/* TODO make this non-blocking */
 	syslog(severity, logmsg);
 
 	pthread_mutex_unlock(&log_mutex);
