@@ -1,5 +1,5 @@
 /*
-  Copyright Red Hat, Inc. 2002-2003
+  Copyright Red Hat, Inc. 2002-2006
   Copyright Mission Critical Linux, Inc. 2000
 
   This program is free software; you can redistribute it and/or modify it
@@ -19,21 +19,24 @@
 */
 //#define DEBUG
 #include <platform.h>
-#include <magma.h>
-#include <magmamsg.h>
 #include <resgroup.h>
 #include <vf.h>
-#include <magma.h>
+#include <message.h>
 #include <ccs.h>
 #include <clulog.h>
+#include <members.h>
 #include <list.h>
 #include <reslist.h>
 #include <assert.h>
 
-#define cm_svccount cm_pad[0] /* Theses are uint8_t size */
-#define cm_svcexcl  cm_pad[1]
+/* Use address field in this because we never use it internally,
+   and there is no extra space in the cman_node_t type.
+   */
 
+#define cn_svccount cn_address.cna_address[0] /* Theses are uint8_t size */
+#define cn_svcexcl  cn_address.cna_address[1]
 
+extern char *res_ops[];
 static int config_version = 0;
 static resource_t *_resources = NULL;
 static resource_rule_t *_rules = NULL;
@@ -41,11 +44,15 @@ static resource_node_t *_tree = NULL;
 static fod_t *_domains = NULL;
 
 pthread_mutex_t config_mutex = PTHREAD_MUTEX_INITIALIZER;
+pthread_mutex_t status_mutex = PTHREAD_MUTEX_INITIALIZER;
 pthread_rwlock_t resource_lock = PTHREAD_RWLOCK_INITIALIZER;
 
+void res_build_name(char *, size_t, resource_t *);
+int get_rg_state_local(char *, rg_state_t *);
+
 
 struct status_arg {
-	int fd;
+	msgctx_t *ctx;
 	int fast;
 };
 
@@ -56,7 +63,7 @@ struct status_arg {
    @see node_should_start
  */
 int
-node_should_start_safe(uint64_t nodeid, cluster_member_list_t *membership,
+node_should_start_safe(uint32_t nodeid, cluster_member_list_t *membership,
 		       char *rg_name)
 {
 	int ret;
@@ -73,24 +80,25 @@ int
 count_resource_groups(cluster_member_list_t *ml)
 {
 	resource_t *res;
-	char *rgname, *val;
+	resource_node_t *node;
+	char rgname[64], *val;
 	int x;
 	rg_state_t st;
-	void *lockp;
-	cluster_member_t *mp;
+	struct dlm_lksb lockp;
+	cman_node_t *mp;
 
 	for (x = 0; x < ml->cml_count; x++) {
-		ml->cml_members[x].cm_svccount = 0;
-		ml->cml_members[x].cm_svcexcl = 0;
+		ml->cml_members[x].cn_svccount = 0;
+		ml->cml_members[x].cn_svcexcl = 0;
 	}
 
 	pthread_rwlock_rdlock(&resource_lock);
 
-	list_do(&_resources, res) {
-		if (res->r_rule->rr_root == 0)
-			continue;
+	list_do(&_tree, node) {
 
-		rgname = res->r_attrs[0].ra_value;
+		res = node->rn_resource;
+
+		res_build_name(rgname, sizeof(rgname), res);
 
 		if (rg_lock(rgname, &lockp) < 0) {
 			clulog(LOG_ERR, "#XX: Unable to obtain cluster "
@@ -102,11 +110,11 @@ count_resource_groups(cluster_member_list_t *ml)
 		if (get_rg_state(rgname, &st) < 0) {
 			clulog(LOG_ERR, "#34: Cannot get status "
 			       "for service %s\n", rgname);
-			rg_unlock(rgname, lockp);
+			rg_unlock(&lockp);
 			continue;
 		}
 
-		rg_unlock(rgname, lockp);
+		rg_unlock(&lockp);
 
 		if (st.rs_state != RG_STATE_STARTED &&
 		     st.rs_state != RG_STATE_STARTING)
@@ -116,18 +124,17 @@ count_resource_groups(cluster_member_list_t *ml)
 		if (!mp)
 			continue;
 
-		++mp->cm_svccount;
+		++mp->cn_svccount;
 
 		val = res_attr_value(res, "exclusive");
 		if (val && ((!strcmp(val, "yes") ||
 				     (atoi(val)>0))) ) {
-			++mp->cm_svcexcl;
+			++mp->cn_svcexcl;
 		}
 
-	} while (!list_done(&_resources, res));
+	} while (!list_done(&_tree, node));
 
 	pthread_rwlock_unlock(&resource_lock);
-
 	return 0;
 }
 
@@ -139,14 +146,14 @@ count_resource_groups(cluster_member_list_t *ml)
    - Failover domain (ordering / restricted policy)
    - Exclusive service policy
  */
-uint64_t
-best_target_node(cluster_member_list_t *allowed, uint64_t owner,
+uint32_t
+best_target_node(cluster_member_list_t *allowed, uint32_t owner,
 		 char *rg_name, int lock)
 {
 	int x;
 	int highscore = 1;
 	int score;
-	uint64_t highnode = owner, nodeid;
+	uint32_t highnode = owner, nodeid;
 	char *val;
 	resource_t *res;
 	int exclusive;
@@ -158,13 +165,13 @@ best_target_node(cluster_member_list_t *allowed, uint64_t owner,
 		pthread_rwlock_unlock(&resource_lock);
 
 	for (x=0; x < allowed->cml_count; x++) {
-		if (allowed->cml_members[x].cm_state != STATE_UP)
+		if (!allowed->cml_members[x].cn_member)
 			continue;
 
-		nodeid = allowed->cml_members[x].cm_id;
+		nodeid = allowed->cml_members[x].cn_nodeid;
 
 		/* Don't allow trying a restart just yet */
-		if (owner != NODE_ID_NONE && nodeid == owner)
+		if (owner != 0 && nodeid == owner)
 			continue;
 		
 		if (lock)
@@ -189,13 +196,13 @@ best_target_node(cluster_member_list_t *allowed, uint64_t owner,
 
 		if (exclusive) {
 
-			if (allowed->cml_members[x].cm_svccount > 0) {
+			if (allowed->cml_members[x].cn_svccount > 0) {
 				/* Definitely not this guy */
 				continue;
 			} else {
 				score += 2;
 			}
-		} else if (allowed->cml_members[x].cm_svcexcl) {
+		} else if (allowed->cml_members[x].cn_svcexcl) {
 			/* This guy has an exclusive resource group.
 			   Can't relocate / failover to him. */
 			continue;
@@ -212,6 +219,42 @@ best_target_node(cluster_member_list_t *allowed, uint64_t owner,
 }
 
 
+int
+check_depend(resource_t *res)
+{
+	char *val;
+	rg_state_t rs;
+
+	val = res_attr_value(res, "depend");
+	if (!val)
+		/* No dependency */
+		return -1;
+
+	if (get_rg_state_local(val, &rs) == 0)
+		return (rs.rs_state == RG_STATE_STARTED);
+
+	return 1;
+}
+
+
+int
+check_depend_safe(char *rg_name)
+{
+	resource_t *res;
+	int ret;
+
+	pthread_rwlock_rdlock(&resource_lock);
+	res = find_root_by_ref(&_resources, rg_name);
+	if (!res)
+		return -1;
+
+	ret = check_depend(res);
+	pthread_rwlock_unlock(&resource_lock);
+
+	return ret;
+}
+
+
 /**
   Start or failback a resource group: if it's not running, start it.
   If it is running and we're a better member to run it, then ask for
@@ -222,9 +265,9 @@ consider_start(resource_node_t *node, char *svcName, rg_state_t *svcStatus,
 	       cluster_member_list_t *membership)
 {
 	char *val;
-	cluster_member_t *mp;
+	cman_node_t *mp;
 	int autostart, exclusive;
-	void *lockp;
+	struct dlm_lksb lockp;
 
 	mp = memb_id_to_p(membership, my_id());
 	assert(mp);
@@ -234,7 +277,7 @@ consider_start(resource_node_t *node, char *svcName, rg_state_t *svcStatus,
 	 * local start.
 	 */
 	if (svcStatus->rs_state == RG_STATE_STARTED &&
-	    svcStatus->rs_owner == mp->cm_id)
+	    svcStatus->rs_owner == mp->cn_nodeid)
 		return;
 
 	if (svcStatus->rs_state == RG_STATE_DISABLED)
@@ -267,7 +310,7 @@ consider_start(resource_node_t *node, char *svcName, rg_state_t *svcStatus,
 			if (get_rg_state(svcName, svcStatus) != 0) {
 				clulog(LOG_ERR, "#34: Cannot get status "
 				       "for service %s\n", svcName);
-				rg_unlock(svcName, lockp);
+				rg_unlock(&lockp);
 				return;
 			}
 
@@ -277,16 +320,24 @@ consider_start(resource_node_t *node, char *svcName, rg_state_t *svcStatus,
 				set_rg_state(svcName, svcStatus);
 			}
 
-			rg_unlock(svcName, lockp);
+			rg_unlock(&lockp);
 
 			return;
 		}
 	}
 
+	/* See if service this one depends on is running.  If not,
+           don't start it */
+	if (check_depend(node->rn_resource) == 0) {
+		clulog(LOG_DEBUG,
+		       "Skipping RG %s: Dependency missing\n", svcName);
+		return;
+	}
+
 	val = res_attr_value(node->rn_resource, "exclusive");
 	exclusive = val && ((!strcmp(val, "yes") || (atoi(val)>0)));
 
-	if (exclusive && mp->cm_svccount) {
+	if (exclusive && mp->cn_svccount) {
 		clulog(LOG_DEBUG,
 		       "Skipping RG %s: Exclusive and I am running services\n",
 		       svcName);
@@ -297,7 +348,7 @@ consider_start(resource_node_t *node, char *svcName, rg_state_t *svcStatus,
 	   Don't start other services if I'm running an exclusive
 	   service.
 	 */
-	if (mp->cm_svcexcl) {
+	if (mp->cn_svcexcl) {
 		clulog(LOG_DEBUG,
 		       "Skipping RG %s: I am running an exclusive service\n",
 		       svcName);
@@ -308,15 +359,15 @@ consider_start(resource_node_t *node, char *svcName, rg_state_t *svcStatus,
 	 * Start any stopped services, or started services
 	 * that are owned by a down node.
 	 */
-	if (node_should_start(mp->cm_id, membership, svcName, &_domains) ==
+	if (node_should_start(mp->cn_nodeid, membership, svcName, &_domains) ==
 	    FOD_BEST)
-		rt_enqueue_request(svcName, RG_START, -1, 0, mp->cm_id,
+		rt_enqueue_request(svcName, RG_START, NULL, 0, mp->cn_nodeid,
 				   0, 0);
 }
 
 
 void
-consider_relocate(char *svcName, rg_state_t *svcStatus, uint64_t nodeid,
+consider_relocate(char *svcName, rg_state_t *svcStatus, uint32_t nodeid,
 		  cluster_member_list_t *membership)
 {
 	int a, b;
@@ -349,7 +400,7 @@ consider_relocate(char *svcName, rg_state_t *svcStatus, uint64_t nodeid,
 	       svcName,
 	       memb_id_to_name(membership, nodeid));
 
-	rt_enqueue_request(svcName, RG_RELOCATE, -1, 0, nodeid, 0, 0);
+	rt_enqueue_request(svcName, RG_RELOCATE, NULL, 0, nodeid, 0, 0);
 }
 
 
@@ -361,17 +412,17 @@ consider_relocate(char *svcName, rg_state_t *svcStatus, uint64_t nodeid,
  * @see			node_event
  */
 int
-eval_groups(int local, uint64_t nodeid, int nodeStatus)
+eval_groups(int local, uint32_t nodeid, int nodeStatus)
 {
-	void *lockp = NULL;
-	char *svcName, *nodeName;
+	struct dlm_lksb lockp;
+	char svcName[64], *nodeName;
 	resource_node_t *node;
 	rg_state_t svcStatus;
 	cluster_member_list_t *membership;
 	int ret;
 
 	if (rg_locked()) {
-		clulog(LOG_NOTICE,
+		clulog(LOG_DEBUG,
 			"Resource groups locked; not evaluating\n");
 		return -EAGAIN;
 	}
@@ -385,10 +436,7 @@ eval_groups(int local, uint64_t nodeid, int nodeStatus)
 
 	list_do(&_tree, node) {
 
-		if (node->rn_resource->r_rule->rr_root == 0)
-			continue;
-
-		svcName = node->rn_resource->r_attrs->ra_value;
+		res_build_name(svcName, sizeof(svcName), node->rn_resource);
 
 		/*
 		 * Lock the service information and get the current service
@@ -399,7 +447,7 @@ eval_groups(int local, uint64_t nodeid, int nodeStatus)
 			       "#33: Unable to obtain cluster lock: %s\n",
 			       strerror(-ret));
 			pthread_rwlock_unlock(&resource_lock);
-			cml_free(membership);
+			free_member_list(membership);
 			return ret;
 		}
 		
@@ -407,13 +455,13 @@ eval_groups(int local, uint64_t nodeid, int nodeStatus)
 			clulog(LOG_ERR,
 			       "#34: Cannot get status for service %s\n",
 			       svcName);
-			rg_unlock(svcName, lockp);
+			rg_unlock(&lockp);
 			continue;
 		}
 
-		rg_unlock(svcName, lockp);
+		rg_unlock(&lockp);
 
-		if (svcStatus.rs_owner == NODE_ID_NONE)
+		if (svcStatus.rs_owner == 0)
 			nodeName = "none";
 		else
 			nodeName = memb_id_to_name(membership,
@@ -431,12 +479,12 @@ eval_groups(int local, uint64_t nodeid, int nodeStatus)
 		       rg_state_str(svcStatus.rs_state),
 		       nodeName);
 
-		if ((local && (nodeStatus == STATE_UP)) ||
+		if ((local && nodeStatus) ||
 		    svcStatus.rs_state == RG_STATE_STOPPED) {
 
 			consider_start(node, svcName, &svcStatus, membership);
 
-		} else if (!local && (nodeStatus == STATE_DOWN)) {
+		} else if (!local && !nodeStatus) {
 
 			/*
 			 * Start any stopped services, or started services
@@ -459,7 +507,7 @@ eval_groups(int local, uint64_t nodeid, int nodeStatus)
 	} while (!list_done(&_tree, node));
 
 	pthread_rwlock_unlock(&resource_lock);
-	cml_free(membership);
+	free_member_list(membership);
 
 	clulog(LOG_DEBUG, "Event (%d:%d:%d) Processed\n", local,
 	       (int)nodeid, nodeStatus);
@@ -468,6 +516,155 @@ eval_groups(int local, uint64_t nodeid, int nodeStatus)
 }
 
 
+/**
+ * Called to decide what services to start locally after a service event.
+ * 
+ * @see			eval_groups
+ */
+int
+group_event(char *rg_name, uint32_t state, int owner)
+{
+	char svcName[64], *nodeName;
+	resource_node_t *node;
+	rg_state_t svcStatus;
+	cluster_member_list_t *membership;
+	int depend;
+
+	if (rg_locked()) {
+		clulog(LOG_DEBUG,
+			"Resource groups locked; not evaluating\n");
+		return -EAGAIN;
+	}
+
+	membership = member_list();
+	if (!membership)
+		return -1;
+
+	pthread_rwlock_rdlock(&resource_lock);
+
+	/* Requires read lock */
+	count_resource_groups(membership);
+
+	list_do(&_tree, node) {
+
+		res_build_name(svcName, sizeof(svcName), node->rn_resource);
+
+		if (get_rg_state_local(svcName, &svcStatus) != 0)
+			continue;
+
+		if (svcStatus.rs_owner == 0)
+			nodeName = "none";
+		else
+			nodeName = memb_id_to_name(membership,
+						   svcStatus.rs_owner);
+
+		/* Disabled/failed/in recovery?  Do nothing */
+		if ((svcStatus.rs_state == RG_STATE_DISABLED) ||
+		    (svcStatus.rs_state == RG_STATE_FAILED) ||
+		    (svcStatus.rs_state == RG_STATE_RECOVER)) {
+			continue;
+		}
+
+		depend = check_depend(node->rn_resource);
+
+		/* Skip if we have no dependency */
+		if (depend == -1)
+			continue;
+
+		/*
+		   If we have:
+		   (a) a met dependency
+		   (b) we're in the STOPPED state, and
+		   (c) our new service event is a started service
+
+		   Then see if we should start this other service as well.
+		 */
+		if (depend == 1 &&
+		    svcStatus.rs_state == RG_STATE_STOPPED &&
+		    state == RG_STATE_STARTED) {
+
+			clulog(LOG_DEBUG, "Evaluating RG %s, state %s, owner "
+			       "%s\n", svcName,
+			       rg_state_str(svcStatus.rs_state),
+			       nodeName);
+			consider_start(node, svcName, &svcStatus, membership);
+			continue;
+		}
+		
+		/*
+		   If we lost a dependency for this service and it's running
+		   locally, stop it.
+		 */
+		if (depend == 0 &&
+		    svcStatus.rs_state == RG_STATE_STARTED &&
+		    svcStatus.rs_owner == my_id()) {
+
+			clulog(LOG_WARNING, "Stopping service %s: Dependency missing\n",
+			       svcName);
+			rt_enqueue_request(svcName, RG_STOP, NULL, 0, my_id(),
+					   0, 0);
+		}
+
+	} while (!list_done(&_tree, node));
+
+	pthread_rwlock_unlock(&resource_lock);
+	free_member_list(membership);
+
+	return 0;
+}
+
+
+/**
+  Tells us if a resource group can be migrated.
+ */
+int
+group_migratory(char *groupname)
+{
+	resource_node_t *rn;
+	resource_t *res;
+	int migrate = 0, x;
+
+	pthread_rwlock_rdlock(&resource_lock);
+
+	res = find_root_by_ref(&_resources, groupname);
+	if (!res) {
+		pthread_rwlock_unlock(&resource_lock);
+		/* Nonexistent or non-TL RG cannot be migrated */
+		return 0;
+	}
+
+	for (x = 0; res->r_rule->rr_actions[x].ra_name; x++) {
+		if (!strcmp(res->r_rule->rr_actions[x].ra_name,
+		    "migrate")) {
+			migrate = 1;
+			break;
+		}
+	}
+
+	if (!migrate) {
+		pthread_rwlock_unlock(&resource_lock);
+		/* resource rule missing 'migrate' command */
+		return 0;
+	}
+
+	list_do(&_tree, rn) {
+		if (rn->rn_resource == res && rn->rn_child) {
+			pthread_rwlock_unlock(&resource_lock);
+			/* TL service w/ children cannot be migrated */
+			return 0;
+		}
+	} while (!list_done(&_tree, rn));
+
+	pthread_rwlock_unlock(&resource_lock);
+
+	/* Ok, we have a migrate option to the resource group,
+	   the resource group has no children, and the resource
+	   group exists.  We're all good */
+	return 1;
+}
+
+
+
 /**
    Perform an operation on a resource group.  That is, walk down the
    tree for that resource group, performing the given operation on
@@ -534,6 +731,38 @@ group_op(char *groupname, int op)
 }
 
 
+int
+group_migrate(char *groupname, int target)
+{
+	resource_t *res;
+	char *tgt_name;
+	int ret = RG_ENOSERVICE;
+	cluster_member_list_t *membership;
+
+	membership = member_list();
+	if (!membership)
+		return RG_EFAIL;
+
+	pthread_rwlock_rdlock(&resource_lock);
+	
+	tgt_name = memb_id_to_name(membership, target);
+	res = find_root_by_ref(&_resources, groupname);
+	if (!res)
+		goto out;
+
+	if (!tgt_name) {
+		ret = RG_EINVAL;
+		goto out;
+	}
+	ret = res_exec(res, res_ops[RG_MIGRATE], tgt_name);
+
+out:
+	pthread_rwlock_unlock(&resource_lock);
+	free_member_list(membership);
+	return ret;
+}
+
+
 /**
    Gets an attribute of a resource group.
 
@@ -576,12 +805,11 @@ group_property(char *groupname, char *property, char *ret, size_t len)
   @param rgname		Resource group name whose state we want to send.
   @see send_rg_states
  */
-int get_rg_state_local(char *, rg_state_t *);
 void
-send_rg_state(int fd, char *rgname, int fast)
+send_rg_state(msgctx_t *ctx, char *rgname, int fast)
 {
 	rg_state_msg_t msg, *msgp = &msg;
-	void *lockp;
+	struct dlm_lksb lockp;
 
 	msgp->rsm_hdr.gh_magic = GENERIC_HDR_MAGIC;
 	msgp->rsm_hdr.gh_length = sizeof(msg);
@@ -594,15 +822,15 @@ send_rg_state(int fd, char *rgname, int fast)
 		if (rg_lock(rgname, &lockp) < 0)
 			return;
 		if (get_rg_state(rgname, &msgp->rsm_state) < 0) {
-			rg_unlock(rgname, lockp);
+			rg_unlock(&lockp);
 			return;
 		}
-		rg_unlock(rgname, lockp);
+		rg_unlock(&lockp);
 	}
 
 	swab_rg_state_msg_t(msgp);
 
-	if (msg_send(fd, msgp, sizeof(msg)) < 0)
+	if (msg_send(ctx, msgp, sizeof(msg)) < 0)
 		perror("msg_send");
 }
 
@@ -614,31 +842,42 @@ send_rg_state(int fd, char *rgname, int fast)
 static void *
 status_check_thread(void *arg)
 {
-	int fd = ((struct status_arg *)arg)->fd;
+	msgctx_t *ctx = ((struct status_arg *)arg)->ctx;
 	int fast = ((struct status_arg *)arg)->fast;
-	resource_t *res;
+	resource_node_t *node;
 	generic_msg_hdr hdr;
+	char rg[64];
 
 	free(arg);
 
+	/* See if we have a slot... */
+	if (rg_inc_status() < 0) {
+		/* Too many outstanding status checks.  try again later. */
+		msg_send_simple(ctx, RG_FAIL, RG_EAGAIN, 0);
+		msg_close(ctx);
+		msg_free_ctx(ctx);
+		return NULL;
+	}
+	
 	pthread_rwlock_rdlock(&resource_lock);
 
-	list_do(&_resources, res) {
-		if (res->r_rule->rr_root == 0)
-			continue;
+	list_do(&_tree, node) {
 
-		send_rg_state(fd, res->r_attrs[0].ra_value, fast);
-	} while (!list_done(&_resources, res));
+		res_build_name(rg, sizeof(rg), node->rn_resource);
+		send_rg_state(ctx, rg, fast);
+	} while (!list_done(&_tree, node));
 
 	pthread_rwlock_unlock(&resource_lock);
 
-	msg_send_simple(fd, RG_SUCCESS, 0, 0);
+	msg_send_simple(ctx, RG_SUCCESS, 0, 0);
 
 	/* XXX wait for client to tell us it's done; I don't know why
 	   this is needed when doing fast I/O, but it is. */
-	msg_receive_timeout(fd, &hdr, sizeof(hdr), 10);
-
-	msg_close(fd);
+	msg_receive(ctx, &hdr, sizeof(hdr), 10);
+	msg_close(ctx);
+	msg_free_ctx(ctx);
+	
+	rg_dec_status();
 
 	return NULL;
 }
@@ -651,7 +890,7 @@ status_check_thread(void *arg)
   @return		0
  */
 int
-send_rg_states(int fd, int fast)
+send_rg_states(msgctx_t *ctx, int fast)
 {
 	struct status_arg *arg;
 	pthread_t newthread;
@@ -659,11 +898,11 @@ send_rg_states(int fd, int fast)
 
 	arg = malloc(sizeof(struct status_arg));
 	if (!arg) {
-		msg_send_simple(fd, RG_FAIL, 0, 0);
+		msg_send_simple(ctx, RG_FAIL, 0, 0);
 		return -1;
 	}
 
-	arg->fd = fd;
+	arg->ctx = ctx;
 	arg->fast = fast;
 
         pthread_attr_init(&attrs);
@@ -681,21 +920,20 @@ send_rg_states(int fd, int fast)
 int
 svc_exists(char *svcname)
 {
-	resource_t *res;
+	resource_node_t *node;
 	int ret = 0;
+	char rg[64];
 
 	pthread_rwlock_rdlock(&resource_lock);
 
-	list_do(&_resources, res) {
-		if (res->r_rule->rr_root == 0)
-			continue;
+	list_do(&_tree, node) {
+		res_build_name(rg, sizeof(rg), node->rn_resource);
 
-		if (strcmp(res->r_attrs[0].ra_value, 
-			   svcname) == 0) {
+		if (strcmp(rg, svcname) == 0) {
 			ret = 1;
 			break;
 		}
-	} while (!list_done(&_resources, res));
+	} while (!list_done(&_tree, node));
 
 	pthread_rwlock_unlock(&resource_lock);
 
@@ -707,25 +945,22 @@ void
 rg_doall(int request, int block, char *debugfmt)
 {
 	resource_node_t *curr;
-	char *name;
 	rg_state_t svcblk;
+	char rg[64];
 
 	pthread_rwlock_rdlock(&resource_lock);
 	list_do(&_tree, curr) {
 
-		if (curr->rn_resource->r_rule->rr_root == 0)
-			continue;
-
 		/* Group name */
-		name = curr->rn_resource->r_attrs->ra_value;
+		res_build_name(rg, sizeof(rg), curr->rn_resource);
 
 		if (debugfmt)
-			clulog(LOG_DEBUG, debugfmt, name);
+			clulog(LOG_DEBUG, debugfmt, rg);
 
 		/* Optimization: Don't bother even queueing the request
 		   during the exit case if we don't own it */
 		if (request == RG_STOP_EXITING) {
-			if (get_rg_state_local(name, &svcblk) < 0)
+			if (get_rg_state_local(rg, &svcblk) < 0)
 				continue;
 
 			/* Always run stop if we're the owner, regardless
@@ -734,8 +969,8 @@ rg_doall(int request, int block, char *debugfmt)
 				continue;
 		}
 
-		rt_enqueue_request(name, request, -1, 0,
-				   NODE_ID_NONE, 0, 0);
+		rt_enqueue_request(rg, request, NULL, 0,
+				   0, 0, 0);
 	} while (!list_done(&_tree, curr));
 
 	pthread_rwlock_unlock(&resource_lock);
@@ -755,20 +990,21 @@ void *
 q_status_checks(void *arg)
 {
 	resource_node_t *curr;
-	char *name;
 	rg_state_t svcblk;
+	char rg[64];
+	
+	/* Only one status thread at a time, please! */
+	if (pthread_mutex_trylock(&status_mutex) != 0)
+		return NULL;
 
 	pthread_rwlock_rdlock(&resource_lock);
 	list_do(&_tree, curr) {
 
-		if (curr->rn_resource->r_rule->rr_root == 0)
-			continue;
-
 		/* Group name */
-		name = curr->rn_resource->r_attrs->ra_value;
+		res_build_name(rg, sizeof(rg), curr->rn_resource);
 
 		/* Local check - no one will make us take a service */
-		if (get_rg_state_local(name, &svcblk) < 0) {
+		if (get_rg_state_local(rg, &svcblk) < 0) {
 			continue;
 		}
 
@@ -776,12 +1012,13 @@ q_status_checks(void *arg)
 		    svcblk.rs_state != RG_STATE_STARTED)
 			continue;
 
-		rt_enqueue_request(name, RG_STATUS,
-				   -1, 0, NODE_ID_NONE, 0, 0);
+		rt_enqueue_request(rg, RG_STATUS,
+				   NULL, 0, 0, 0, 0);
 
 	} while (!list_done(&_tree, curr));
 
 	pthread_rwlock_unlock(&resource_lock);
+	pthread_mutex_unlock(&status_mutex);
 
 	return NULL;
 }
@@ -810,23 +1047,20 @@ void
 do_condstops(void)
 {
 	resource_node_t *curr;
-	char *name;
 	rg_state_t svcblk;
 	int need_kill;
+	char rg[64];
 
 	clulog(LOG_INFO, "Stopping changed resources.\n");
 
 	pthread_rwlock_rdlock(&resource_lock);
 	list_do(&_tree, curr) {
 
-		if (curr->rn_resource->r_rule->rr_root == 0)
-			continue;
-
 		/* Group name */
-		name = curr->rn_resource->r_attrs->ra_value;
+		res_build_name(rg, sizeof(rg), curr->rn_resource);
 
 		/* If we're not running it, no need to CONDSTOP */
-		if (get_rg_state_local(name, &svcblk) < 0) {
+		if (get_rg_state_local(rg, &svcblk) < 0) {
 			continue;
 		}
 
@@ -837,11 +1071,11 @@ do_condstops(void)
 		need_kill = 0;
 		if (curr->rn_resource->r_flags & RF_NEEDSTOP) {
 			need_kill = 1;
-			clulog(LOG_DEBUG, "Removing %s\n", name);
+			clulog(LOG_DEBUG, "Removing %s\n", rg);
 		}
 
-		rt_enqueue_request(name, need_kill ? RG_DISABLE : RG_CONDSTOP,
-				   -1, 0, NODE_ID_NONE, 0, 0);
+		rt_enqueue_request(rg, need_kill ? RG_DISABLE : RG_CONDSTOP,
+				   NULL, 0, 0, 0, 0);
 
 	} while (!list_done(&_tree, curr));
 
@@ -857,10 +1091,10 @@ void
 do_condstarts(void)
 {
 	resource_node_t *curr;
-	char *name, *val;
+	char rg[64], *val;
 	rg_state_t svcblk;
 	int need_init, new_groups = 0, autostart;
-	void *lockp;
+	struct dlm_lksb lockp;
 
 	clulog(LOG_INFO, "Starting changed resources.\n");
 
@@ -868,11 +1102,8 @@ do_condstarts(void)
 	pthread_rwlock_rdlock(&resource_lock);
 	list_do(&_tree, curr) {
 
-		if (curr->rn_resource->r_rule->rr_root == 0)
-			continue;
-
 		/* Group name */
-		name = curr->rn_resource->r_attrs->ra_value;
+		res_build_name(rg, sizeof(rg), curr->rn_resource);
 
 		/* New RG.  We'll need to initialize it. */
 		need_init = 0;
@@ -880,31 +1111,30 @@ do_condstarts(void)
 			need_init = 1;
 
 		if (!need_init) {
-			if (get_rg_state_local(name, &svcblk) < 0)
+			if (get_rg_state_local(rg, &svcblk) < 0)
 				continue;
 		} else {
-			if (rg_lock(name, &lockp) != 0)
+			if (rg_lock(rg, &lockp) != 0)
 				continue;
 
-			if (get_rg_state(name, &svcblk) < 0) {
-				rg_unlock(name, lockp);
+			if (get_rg_state(rg, &svcblk) < 0) {
+				rg_unlock(&lockp);
 				continue;
 			}
 
-			rg_unlock(name, lockp);
+			rg_unlock(&lockp);
 		}
 
-		if (!need_init && svcblk.rs_owner != my_id()) {
+		if (!need_init && svcblk.rs_owner != my_id())
 			continue;
-		}
 
 		if (need_init) {
 			++new_groups;
-			clulog(LOG_DEBUG, "Initializing %s\n", name);
+			clulog(LOG_DEBUG, "Initializing %s\n", rg);
 		}
 
-		rt_enqueue_request(name, need_init ? RG_INIT : RG_CONDSTART,
-				   -1, 0, NODE_ID_NONE, 0, 0);
+		rt_enqueue_request(rg, need_init ? RG_INIT : RG_CONDSTART,
+				   NULL, 0, 0, 0, 0);
 
 	} while (!list_done(&_tree, curr));
 
@@ -918,21 +1148,18 @@ do_condstarts(void)
 	pthread_rwlock_rdlock(&resource_lock);
 	list_do(&_tree, curr) {
 
-		if (curr->rn_resource->r_rule->rr_root == 0)
-			continue;
-
 		/* Group name */
-		name = curr->rn_resource->r_attrs->ra_value;
+		res_build_name(rg, sizeof(rg), curr->rn_resource);
 
 		/* New RG.  We'll need to initialize it. */
 		if (!(curr->rn_resource->r_flags & RF_NEEDSTART))
 			continue;
 
-		if (rg_lock(name, &lockp) != 0)
+		if (rg_lock(rg, &lockp) != 0)
 			continue;
 
-		if (get_rg_state(name, &svcblk) < 0) {
-			rg_unlock(name, lockp);
+		if (get_rg_state(rg, &svcblk) < 0) {
+			rg_unlock(&lockp);
 			continue;
 		}
 
@@ -942,7 +1169,7 @@ do_condstarts(void)
 		   a truly new service, it will be in the UNINITIALIZED
 		   state, which will be caught by eval_groups. */
 		if (svcblk.rs_state != RG_STATE_DISABLED) {
-			rg_unlock(name, lockp);
+			rg_unlock(&lockp);
 			continue;
 		}
 
@@ -955,15 +1182,15 @@ do_condstarts(void)
 		else
 			svcblk.rs_state = RG_STATE_DISABLED;
 
-		set_rg_state(name, &svcblk);
+		set_rg_state(rg, &svcblk);
 
-		rg_unlock(name, lockp);
+		rg_unlock(&lockp);
 
 	} while (!list_done(&_tree, curr));
 	pthread_rwlock_unlock(&resource_lock);
 
 	/* Pass 3: See if we should start new resource groups */
-	eval_groups(1, my_id(), STATE_UP);
+	eval_groups(1, my_id(), 1);
 }
 
 
@@ -1033,6 +1260,13 @@ init_resource_groups(int reconfigure)
 		pthread_mutex_lock(&config_mutex);
 		config_version = atoi(val);
 		pthread_mutex_unlock(&config_mutex);
+		free(val);
+	}
+	
+	if (ccs_get(fd, "/cluster/rm/@statusmax", &val) == 0) {
+		if (strlen(val)) 
+			rg_set_statusmax(atoi(val));
+		free(val);
 	}
 
 	clulog(LOG_DEBUG, "Building Resource Trees\n");
