@@ -1,5 +1,5 @@
 /*
-  Copyright Red Hat, Inc. 2004
+  Copyright Red Hat, Inc. 2004-2006
 
   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
@@ -73,7 +73,8 @@ const char *res_ops[] = {
 	"condstop",
 	"monitor",
 	"meta-data",		/* printenv */
-	"validate-all"
+	"validate-all",
+	"migrate"
 };
 
 
@@ -220,14 +221,13 @@ add_ocf_stuff(resource_t *res, char **env, int depth)
    Allocate and fill an environment variable array.
 
    @param node		Node in resource tree to use for parameters
-   @param op		Operation (start/stop/status/monitor/etc.)
    @param depth		Depth (status/monitor/etc.)
    @return		Newly allocated environment array or NULL if
    			one could not be formed.
    @see			kill_env res_exec add_ocf_stuff
  */
 static char **
-build_env(resource_node_t *node, int op, int depth)
+build_env(resource_node_t *node, int depth)
 {
 	resource_t *res = node->rn_resource;
 	char **env;
@@ -332,7 +332,7 @@ restore_signals(void)
    @see			build_env
  */
 int
-res_exec(resource_node_t *node, int op, int depth)
+res_exec(resource_node_t *node, const char *op, const char *arg, int depth)
 {
 	int childpid, pid;
 	int ret = 0;
@@ -340,30 +340,18 @@ res_exec(resource_node_t *node, int op, int depth)
 	resource_t *res = node->rn_resource;
 	char fullpath[2048];
 
-	if (!res->r_rule->rr_agent) {
-		clulog(LOG_DEBUG,
-		       "%s on %s \"%s\" no rr_agent\n",
-		       res_ops[op], res->r_rule->rr_type,
-		       res->r_attrs->ra_value);
+	if (!res->r_rule->rr_agent)
 		return 0;
-	}
 
 #ifdef DEBUG
-	env = build_env(node, op);
-	if (!env) {
-		clulog(LOG_DEBUG,
-		       "%s on %s \"%s\" build_env failed %d\n",
-		       res_ops[op], res->r_rule->rr_type,
-		       res->r_attrs->ra_value, errno);
+	env = build_env(node, depth);
+	if (!env)
 		return -errno;
-	}
 #endif
 
 	childpid = fork();
-	if (childpid < 0) {
-		clulog(LOG_ERR, "%s: fork failed (%d)!\n", __func__, errno);
+	if (childpid < 0)
 		return -errno;
-	}
 
 	if (!childpid) {
 		/* Child */ 
@@ -377,16 +365,11 @@ res_exec(resource_node_t *node, int op, int depth)
 #endif
 
 #ifndef DEBUG
-		env = build_env(node, op, depth);
+		env = build_env(node, depth);
 #endif
 
-		if (!env) {
-			clulog(LOG_DEBUG,
-		       		"%s on %s \"%s\" build_env failed (ENOMEM)\n",
-		       		res_ops[op], res->r_rule->rr_type,
-		       		res->r_attrs->ra_value);
+		if (!env)
 			exit(-ENOMEM);
-		}
 
 		if (res->r_rule->rr_agent[0] != '/')
 			snprintf(fullpath, sizeof(fullpath), "%s/%s",
@@ -397,7 +380,10 @@ res_exec(resource_node_t *node, int op, int depth)
 
 		restore_signals();
 
-		execle(fullpath, fullpath, res_ops[op], NULL, env);
+		if (arg)
+			execle(fullpath, fullpath, op, arg, NULL, env);
+		else
+			execle(fullpath, fullpath, op, NULL, env);
 	}
 
 #ifdef DEBUG
@@ -416,14 +402,8 @@ res_exec(resource_node_t *node, int op, int depth)
 
 		if (ret) {
 			clulog(LOG_NOTICE,
-			       "%s on %s:%s returned %d (%s)\n",
-			       res_ops[op], res->r_rule->rr_type,
-			       res->r_attrs->ra_value, ret,
-			       ocf_strerror(ret));
-		} else {
-			clulog(LOG_DEBUG,
-			       "%s on %s:%s returned %d (%s)\n",
-			       res_ops[op], res->r_rule->rr_type,
+			       "%s on %s \"%s\" returned %d (%s)\n",
+			       op, res->r_rule->rr_type,
 			       res->r_attrs->ra_value, ret,
 			       ocf_strerror(ret));
 		}
@@ -434,11 +414,6 @@ res_exec(resource_node_t *node, int op, int depth)
 	if (!WIFSIGNALED(ret))
 		assert(0);
 
-	clulog(LOG_ERR,
-	       "%s on %s:%s caught signal %d\n",
-	       res_ops[op], res->r_rule->rr_type,
-	       res->r_attrs->ra_value, WTERMSIG(ret));
-
 	return -EFAULT;
 }
 
@@ -617,9 +592,6 @@ build_resource_tree(int ccsfd, resource_node_t **tree,
 	/* Find and build the list of root nodes */
 	list_do(rulelist, curr) {
 
-		if (!curr->rr_root)
-			continue;
-
 		build_tree(ccsfd, &root, NULL, curr, rulelist, reslist, tok);
 
 	} while (!list_done(rulelist, curr));
@@ -660,6 +632,7 @@ _print_resource_tree(resource_node_t **tree, int level)
 {
 	resource_node_t *node;
 	int x, y;
+	char *val;
 
 	list_do(tree, node) {
 		for (x = 0; x < level; x++)
@@ -682,11 +655,14 @@ _print_resource_tree(resource_node_t **tree, int level)
 		     node->rn_resource->r_attrs[x].ra_value; x++) {
 			for (y = 0; y < level+1; y++)
 				printf("  ");
+
+			val = attr_value(node,
+					 node->rn_resource->r_attrs[x].ra_name);
+			if (!val &&
+			    node->rn_resource->r_attrs[x].ra_flags&RA_INHERIT)
+				continue;
 			printf("%s = \"%s\";\n",
-			       node->rn_resource->r_attrs[x].ra_name,
-			       attr_value(node,
-					  node->rn_resource->r_attrs[x].ra_name)
-			      );
+			       node->rn_resource->r_attrs[x].ra_name, val);
 		}
 
 		_print_resource_tree(&node->rn_child, level + 1);
@@ -880,7 +856,7 @@ do_status(resource_node_t *node)
 		return 0;
 
        node->rn_actions[idx].ra_last = now;
-	if ((x = res_exec(node, RS_STATUS,
+	if ((x = res_exec(node, res_ops[RS_STATUS], NULL,
                          node->rn_actions[idx].ra_depth)) == 0)
 		return 0;
 
@@ -888,7 +864,7 @@ do_status(resource_node_t *node)
 		return x;
 
 	/* Strange/failed status. Try to recover inline. */
-	if ((x = res_exec(node, RS_RECOVER, 0)) == 0)
+	if ((x = res_exec(node, res_ops[RS_RECOVER], NULL, 0)) == 0)
 		return 0;
 
 	return x;
@@ -1038,7 +1014,7 @@ _res_op(resource_node_t **tree, resource_t *first,
 		if (me && (op == RS_START)) {
 			node->rn_flags &= ~RF_NEEDSTART;
 
-			rv = res_exec(node, op, 0);
+			rv = res_exec(node, res_ops[op], NULL, 0);
 			if (rv != 0) {
 				node->rn_state = RES_FAILED;
 				return rv;
@@ -1051,18 +1027,24 @@ _res_op(resource_node_t **tree, resource_t *first,
 				++node->rn_resource->r_incarnations;
 				node->rn_state = RES_STARTED;
 			}
+		} else if (me && (op == RS_STATUS)) {
+			/* Check status before children*/
+			rv = do_status(node);
+			if (rv != 0)
+				return rv;
 		}
 
+
 		if (node->rn_child) {
 			rv = _res_op_by_level(&node, me?NULL:first, ret, op);
 			if (rv != 0)
 				return rv;
 		}
 
-		/* Stop/status/etc stops after children have stopped */
+		/* Stop should occur after children have stopped */
 		if (me && (op == RS_STOP)) {
 			node->rn_flags &= ~RF_NEEDSTOP;
-			rv = res_exec(node, op, 0);
+			rv = res_exec(node, res_ops[op], NULL, 0);
 
 			if (rv != 0) {
 				node->rn_state = RES_FAILED;
@@ -1073,12 +1055,6 @@ _res_op(resource_node_t **tree, resource_t *first,
 				--node->rn_resource->r_incarnations;
 				node->rn_state = RES_STOPPED;
 			}
-
-		} else if (me && (op == RS_STATUS)) {
-
-			rv = do_status(node);
-			if (rv != 0)
-				return rv;
 		}
 
 		/*
