@@ -19,21 +19,26 @@
 //#define DEBUG
 #include <assert.h>
 #include <platform.h>
-#include <magma.h>
-#include <magmamsg.h>
+#include <message.h>
+#include <members.h>
+#ifdef OPENAIS
+#include <ds.h>
+#else
 #include <vf.h>
+#endif
 #include <stdio.h>
 #include <string.h>
 #include <resgroup.h>
 #include <clulog.h>
+#include <lock.h>
 #include <rg_locks.h>
 #include <ccs.h>
 #include <rg_queue.h>
 #include <msgsimple.h>
 
-int node_should_start_safe(uint64_t, cluster_member_list_t *, char *);
+int node_should_start_safe(uint32_t, cluster_member_list_t *, char *);
 
-uint64_t next_node_id(cluster_member_list_t *membership, uint64_t me);
+int next_node_id(cluster_member_list_t *membership, int me);
 
 int rg_exec_script(char *rgname, char *script, char *action);
 static int _svc_stop_finish(char *svcName, int failed, uint32_t newstate);
@@ -41,17 +46,19 @@ static int _svc_stop_finish(char *svcName, int failed, uint32_t newstate);
 int set_rg_state(char *servicename, rg_state_t *svcblk);
 int get_rg_state(char *servicename, rg_state_t *svcblk);
 void get_recovery_policy(char *rg_name, char *buf, size_t buflen);
+int check_depend_safe(char *servicename);
+int group_migratory(char *servicename);
 
 
-uint64_t
-next_node_id(cluster_member_list_t *membership, uint64_t me)
+int 
+next_node_id(cluster_member_list_t *membership, int me)
 {
-	uint64_t low = (uint64_t)(-1);
-	uint64_t next = me, curr;
+	int low = (int)(-1);
+	int next = me, curr;
 	int x;
 
 	for (x = 0; x < membership->cml_count; x++) {
-		curr = membership->cml_members[x].cm_id;
+		curr = membership->cml_members[x].cn_nodeid;
 		if (curr < low)
 			low = curr;
 
@@ -67,10 +74,35 @@ next_node_id(cluster_member_list_t *membership, uint64_t me)
 }
 
 
+void
+broadcast_event(char *svcName, uint32_t state)
+{
+	SmMessageSt msgp;
+	msgctx_t everyone;
+
+	msgp.sm_hdr.gh_magic = GENERIC_HDR_MAGIC;
+	msgp.sm_hdr.gh_command = RG_EVENT;
+	msgp.sm_hdr.gh_length = sizeof(msgp);
+	msgp.sm_data.d_action = state;
+	strncpy(msgp.sm_data.d_svcName, svcName,
+		sizeof(msgp.sm_data.d_svcName));
+	msgp.sm_data.d_svcOwner = 0;
+	msgp.sm_data.d_ret = 0;
+
+	swab_SmMessageSt(&msgp);
+
+	if (msg_open(MSG_CLUSTER, 0, RG_PORT, &everyone, 0) < 0)
+		return;
+
+	msg_send(&everyone, &msgp, sizeof(msgp));
+	msg_close(&everyone);
+}
+
+
 int
 svc_report_failure(char *svcName)
 {
-	void *lockp = NULL;
+	struct dlm_lksb lockp;
 	rg_state_t svcStatus;
 	char *nodeName;
 	cluster_member_list_t *membership;
@@ -84,10 +116,10 @@ svc_report_failure(char *svcName)
 	if (get_rg_state(svcName, &svcStatus) != 0) {
 		clulog(LOG_ERR, "#42: Couldn't obtain status for RG %s\n",
 		       svcName);
-		clu_unlock(svcName, lockp);
+		rg_unlock(&lockp);
 		return -1;
 	}
-	rg_unlock(svcName, lockp);
+	rg_unlock(&lockp);
 
 	membership = member_list();
 	nodeName = memb_id_to_name(membership, svcStatus.rs_last_owner);
@@ -100,7 +132,7 @@ svc_report_failure(char *svcName)
 		       svcName, (int)svcStatus.rs_last_owner);
 	}
 
-	cml_free(membership);
+	free_member_list(membership);
 
 	clulog(LOG_ALERT,
 	       "#4: Administrator intervention required.\n",
@@ -112,21 +144,21 @@ svc_report_failure(char *svcName)
 
 int
 #ifdef DEBUG
-_rg_lock(char *name, void **p)
+_rg_lock(char *name, struct dlm_lksb *p)
 #else
-rg_lock(char *name, void **p)
+rg_lock(char *name, struct dlm_lksb *p)
 #endif
 {
 	char res[256];
 
-	snprintf(res, sizeof(res), "usrm::rg=\"%s\"", name);
-	return clu_lock(res, CLK_EX, p);
+	snprintf(res, sizeof(res), "rg=\"%s\"", name);
+	return clu_lock(LKM_EXMODE, p, 0, res);
 }
 
 
 #ifdef DEBUG
 int
-_rg_lock_dbg(char *name, void **p, char *file, int line)
+_rg_lock_dbg(char *name, struct dlm_lksb *p, char *file, int line)
 {
 	dprintf("rg_lock(%s) @ %s:%d\n", name, file, line);
 	return _rg_lock(name, p);
@@ -137,33 +169,30 @@ _rg_lock_dbg(char *name, void **p, char *file, int line)
 
 int
 #ifdef DEBUG
-_rg_unlock(char *name, void *p)
+_rg_unlock(struct dlm_lksb *p)
 #else
-rg_unlock(char *name, void *p)
+rg_unlock(struct dlm_lksb *p)
 #endif
 {
-	char res[256];
-
-	snprintf(res, sizeof(res), "usrm::rg=\"%s\"", name);
-	return clu_unlock(res, p);
+	return clu_unlock(p);
 }
 
 
 #ifdef DEBUG
 int
-_rg_unlock_dbg(char *name, void *p, char *file, int line)
+_rg_unlock_dbg(void *p, char *file, int line)
 {
-	dprintf("rg_unlock(%s) @ %s:%d\n", name, file, line);
-	return _rg_unlock(name, p);
+	dprintf("rg_unlock() @ %s:%d\n", file, line);
+	return _rg_unlock(p);
 }
 #endif
 
 
 void
-send_ret(int fd, char *name, int ret, int orig_request, uint64_t newowner)
+send_ret(msgctx_t *ctx, char *name, int ret, int orig_request)
 {
 	SmMessageSt msg, *msgp = &msg;
-	if (fd < 0)
+	if (!ctx)
 		return;
 
 	msgp->sm_hdr.gh_magic = GENERIC_HDR_MAGIC;
@@ -172,27 +201,23 @@ send_ret(int fd, char *name, int ret, int orig_request, uint64_t newowner)
 	msgp->sm_data.d_action = orig_request;
 	strncpy(msgp->sm_data.d_svcName, name,
 		sizeof(msgp->sm_data.d_svcName));
-	if (newowner == NODE_ID_NONE) {
-		msgp->sm_data.d_svcOwner = my_id(); /* XXX Broken */
-	} else { 
-		msgp->sm_data.d_svcOwner = newowner;
-	}
+	msgp->sm_data.d_svcOwner = my_id(); /* XXX Broken */
 	msgp->sm_data.d_ret = ret;
 
 	swab_SmMessageSt(msgp);
-	msg_send(fd, msgp, sizeof(*msgp));
+	msg_send(ctx, msgp, sizeof(*msgp));
 
 	/* :) */
-	msg_close(fd);
+	msg_close(ctx);
 }
 
 	
 void
-send_response(int ret, uint64_t newowner, request_t *req)
+send_response(int ret, int nodeid, request_t *req)
 {
 	SmMessageSt msg, *msgp = &msg;
 
-	if (req->rr_resp_fd < 0)
+	if (req->rr_resp_ctx == NULL)
 		return;
 
 	msgp->sm_hdr.gh_magic = GENERIC_HDR_MAGIC;
@@ -201,46 +226,59 @@ send_response(int ret, uint64_t newowner, request_t *req)
 	msgp->sm_data.d_action = req->rr_orig_request;
 	strncpy(msgp->sm_data.d_svcName, req->rr_group,
 		sizeof(msgp->sm_data.d_svcName));
-	if (newowner == NODE_ID_NONE) {
-		msgp->sm_data.d_svcOwner = my_id(); /* XXX Broken */
-	} else { 
-		msgp->sm_data.d_svcOwner = newowner;
-	}
+	if (!nodeid)
+		msgp->sm_data.d_svcOwner = my_id();
+	else 
+		msgp->sm_data.d_svcOwner = nodeid;
 	msgp->sm_data.d_ret = ret;
 
 	swab_SmMessageSt(msgp);
-	msg_send(req->rr_resp_fd, msgp, sizeof(*msgp));
+	msg_send(req->rr_resp_ctx, msgp, sizeof(*msgp));
 
-	/* :) */
-	msg_close(req->rr_resp_fd);
-	req->rr_resp_fd = -1;
+	/* :( */
+	msg_close(req->rr_resp_ctx);
+	msg_free_ctx(req->rr_resp_ctx);
+	req->rr_resp_ctx = NULL;
 }
 
 
 int
 set_rg_state(char *name, rg_state_t *svcblk)
 {
-	cluster_member_list_t *membership;
 	char res[256];
-	int ret;
+#ifndef OPENAIS
+	cluster_member_list_t *membership;
+	int ret, tries = 0;
+#endif
 
 	if (name)
 		strncpy(svcblk->rs_name, name, sizeof(svcblk->rs_name));
 
-	membership = member_list();
-	snprintf(res, sizeof(res), "usrm::rg=\"%s\"", name);
-	ret = vf_write(membership, VFF_IGN_CONN_ERRORS, res, svcblk,
-       		       sizeof(*svcblk));
-	cml_free(membership);
-	return ret;
+	snprintf(res, sizeof(res), "rg=\"%s\"", name);
+#ifdef OPENAIS
+	if (ds_write(res, svcblk, sizeof(*svcblk)) < 0)
+		return -1;
+	return 0;
+#else
+	do {
+		/* Retry up to 3 times just in case members transition
+		   while we're trying to commit something */
+		membership = member_list();
+		ret = vf_write(membership, VFF_IGN_CONN_ERRORS, res, svcblk,
+       		       	       sizeof(*svcblk));
+		free_member_list(membership);
+	} while (ret == VFR_TIMEOUT && ++tries < 3);
+
+	return (ret==VFR_OK?0:-1);
+#endif
 }
 
 
 static int
 init_rg(char *name, rg_state_t *svcblk)
 {
-	svcblk->rs_owner = NODE_ID_NONE;
-	svcblk->rs_last_owner = NODE_ID_NONE;
+	svcblk->rs_owner = 0;
+	svcblk->rs_last_owner = 0;
 	svcblk->rs_state = RG_STATE_STOPPED;
        	svcblk->rs_restarts = 0;
 	svcblk->rs_transition = 0;	
@@ -255,18 +293,52 @@ get_rg_state(char *name, rg_state_t *svcblk)
 {
 	char res[256];
 	int ret;
-	void *data = NULL;
-	uint32_t datalen = 0;
+#ifdef OPENAIS
+	char data[DS_MIN_SIZE];
+	int datalen;
+#else
 	uint64_t viewno;
+	void *data = NULL;
 	cluster_member_list_t *membership;
+	uint32_t datalen = 0;
+#endif
 
 	/* ... */
 	if (name)
 		strncpy(svcblk->rs_name, name, sizeof(svcblk->rs_name));
 
-	membership = member_list();
+	snprintf(res, sizeof(res),"rg=\"%s\"", svcblk->rs_name);
 
-	snprintf(res, sizeof(res),"usrm::rg=\"%s\"", svcblk->rs_name);
+#ifdef OPENAIS
+	while((datalen = ds_read(res, data, sizeof(data))) < 0) {
+		if (errno == ENOENT) {
+			ds_key_init(res, DS_MIN_SIZE, 10);
+		} else {
+			perror("ds_read");
+			return -1;
+		}
+	}
+
+	if (datalen <= 0) {
+
+		ret = init_rg(name, svcblk);
+		if (ret < 0) {
+			printf("Couldn't initialize rg %s!\n", name);
+			return RG_EFAIL;
+		}
+
+		datalen = ds_read(res, &data, sizeof(data));
+		if (ret < 0) {
+			printf("Couldn't reread rg %s! (%d)\n", name, ret);
+			return RG_EFAIL;
+		}
+	}
+
+	memcpy(svcblk, data, sizeof(*svcblk));
+
+	return 0;
+#else
+	membership = member_list();
 	ret = vf_read(membership, res, &viewno, &data, &datalen);
 
 	if (ret != VFR_OK || datalen == 0) {
@@ -275,36 +347,37 @@ get_rg_state(char *name, rg_state_t *svcblk)
 
 		ret = init_rg(name, svcblk);
 		if (ret != VFR_OK) {
-			cml_free(membership);
+			free_member_list(membership);
 			printf("Couldn't initialize rg %s!\n", name);
-			return FAIL;
+			return RG_EFAIL;
 		}
 
 		ret = vf_read(membership, res, &viewno, &data, &datalen);
 		if (ret != VFR_OK) {
 			if (data)
 				free(data);
-			cml_free(membership);
+			free_member_list(membership);
 			printf("Couldn't reread rg %s! (%d)\n", name, ret);
-			return FAIL;
+			return RG_EFAIL;
 		}
 	}
 
-	if (datalen != sizeof(*svcblk)) {
+	if (datalen < sizeof(*svcblk)) {
 		printf("Size mismatch; expected %d got %d\n",
 		       (int)sizeof(*svcblk), datalen);
 		if (data)
 			free(data);
-		cml_free(membership);
-		return FAIL;
+		free_member_list(membership);
+		return RG_EFAIL;
 	}
 
 	/* Copy out the data. */
 	memcpy(svcblk, data, sizeof(*svcblk));
 	free(data);
-	cml_free(membership);
+	free_member_list(membership);
 
 	return 0;
+#endif
 }
 
 
@@ -314,36 +387,47 @@ get_rg_state_local(char *name, rg_state_t *svcblk)
 {
 	char res[256];
 	int ret;
+#ifdef OPENAIS
+	char data[1024];
+	int datalen;
+#else
 	void *data = NULL;
-	uint32_t datalen = 0;
 	uint64_t viewno;
+	uint32_t datalen;
+#endif
 
 	/* ... */
 	if (name)
 		strncpy(svcblk->rs_name, name, sizeof(svcblk->rs_name));
 
-	snprintf(res, sizeof(res),"usrm::rg=\"%s\"", svcblk->rs_name);
+	snprintf(res, sizeof(res),"rg=\"%s\"", svcblk->rs_name);
+
+#ifdef OPENAIS
+	ret = ds_read(res, data, sizeof(data));
+	if (ret <= 0) {
+#else
 	ret = vf_read_local(res, &viewno, &data, &datalen);
 
 	if (ret != VFR_OK || datalen == 0 ||
 	    datalen != sizeof(*svcblk)) {
 		if (data)
 			free(data);
-
-		svcblk->rs_owner = NODE_ID_NONE;
-		svcblk->rs_last_owner = NODE_ID_NONE;
+#endif
+		svcblk->rs_owner = 0;
+		svcblk->rs_last_owner = 0;
 		svcblk->rs_state = RG_STATE_UNINITIALIZED;
        		svcblk->rs_restarts = 0;
 		svcblk->rs_transition = 0;	
 		strncpy(svcblk->rs_name, name, sizeof(svcblk->rs_name));
 
-		return FAIL;
+		return RG_EFAIL;
 	}
 
 	/* Copy out the data. */
 	memcpy(svcblk, data, sizeof(*svcblk));
+#ifndef OPENAIS
 	free(data);
-
+#endif
 	return 0;
 }
 
@@ -355,7 +439,7 @@ get_rg_state_local(char *name, rg_state_t *svcblk)
  * @param svcStatus	Current service status.
  * @param svcName	Service name
  * @param req		Specify request to perform
- * @return		0 = DO NOT stop service, return FAIL
+ * @return		0 = DO NOT stop service, return RG_EFAIL
  *			1 = STOP service - return whatever it returns.
  *			2 = DO NOT stop service, return 0 (success)
  *                      3 = DO NOT stop service, return RG_EFORWARD
@@ -405,11 +489,11 @@ svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
 			break;
 		}
 
-		if (svcStatus->rs_owner == NODE_ID_NONE ||
+		if (svcStatus->rs_owner == 0 ||
 		    (svcStatus->rs_owner == my_id())) {
 			/*
 			   Service is marked as running locally or on
-			   NODE_ID_NONE (e.g. no member).  Safe
+			   0 (e.g. no member).  Safe
 			   to do a full stop.
 			 */
 			ret = 1;
@@ -464,7 +548,7 @@ svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
 		break;
 	}
 
-	cml_free(membership);
+	free_member_list(membership);
 	return ret;
 }
 
@@ -477,11 +561,11 @@ svc_advise_stop(rg_state_t *svcStatus, char *svcName, int req)
  * @param svcName	Service name
  * @param flags		Specify whether or not it's legal to start a 
  *			disabled service, etc.
- * @return		0 = DO NOT start service, return FAIL
+ * @return		0 = DO NOT start service, return RG_EFAIL
  *			1 = START service - return whatever it returns.
  *			2 = DO NOT start service, return 0
  *			3 = DO NOT start service, return RG_EAGAIN
- *                      4 = DO NOT start servuce, return RG_ERUN
+ *			4 = DO NOT start service, return RG_ERUN
  */
 int
 svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
@@ -524,7 +608,7 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 
 		/* We are allowed to do something with the service.  Make
 		   sure we're not locked */
-		if (svcStatus->rs_owner == NODE_ID_NONE) {
+		if (svcStatus->rs_owner == 0) {
 			if (rg_locked()) {
 				ret = 3;
 				break;
@@ -545,7 +629,7 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 		}
 
 		/*
-		 * Service is running but owner is down -> FAILOVER
+		 * Service is running but owner is down -> RG_EFAILOVER
 		 */
 		clulog(LOG_NOTICE,
 		       "Taking over service %s from down member %s\n",
@@ -615,7 +699,7 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 		break;
 	}
 
-	cml_free(membership);
+	free_member_list(membership);
 	return ret;
 }
 
@@ -631,36 +715,36 @@ svc_advise_start(rg_state_t *svcStatus, char *svcName, int req)
 int
 svc_start(char *svcName, int req)
 {
-	void *lockp = NULL;
+	struct dlm_lksb lockp;
 	int ret;
 	rg_state_t svcStatus;
 
 	if (rg_lock(svcName, &lockp) < 0) {
 		clulog(LOG_ERR, "#45: Unable to obtain cluster lock: %s\n",
 		       strerror(errno));
-		return FAIL;
+		return RG_EFAIL;
 	}
 
 	if (get_rg_state(svcName, &svcStatus) != 0) {
-		rg_unlock(svcName, lockp);
+		rg_unlock(&lockp);
 		clulog(LOG_ERR, "#46: Failed getting status for RG %s\n",
 		       svcName);
-		return FAIL;
+		return RG_EFAIL;
 	}
 
 	/* LOCK HELD */
 	switch (svc_advise_start(&svcStatus, svcName, req)) {
-	case 0: /* Don't start service, return FAIL */
-		rg_unlock(svcName, lockp);
-		return FAIL;
+	case 0: /* Don't start service, return RG_EFAIL */
+		rg_unlock(&lockp);
+		return RG_EFAIL;
 	case 2: /* Don't start service, return 0 */
-		rg_unlock(svcName, lockp);
+		rg_unlock(&lockp);
 		return 0;
 	case 3:
-		rg_unlock(svcName, lockp);
+		rg_unlock(&lockp);
 		return RG_EAGAIN;
 	case 4:
-		rg_unlock(svcName, lockp);
+		rg_unlock(&lockp);
 		return RG_ERUN;
 	default:
 		break;
@@ -677,14 +761,14 @@ svc_start(char *svcName, int req)
 	else
 		svcStatus.rs_restarts = 0;
 
-	if (set_rg_state(svcName, &svcStatus) != 0) {
+	if (set_rg_state(svcName, &svcStatus) < 0) {
 		clulog(LOG_ERR,
 		       "#47: Failed changing service status\n");
-		rg_unlock(svcName, lockp);
-		return FAIL;
+		rg_unlock(&lockp);
+		return RG_EFAIL;
 	}
 	
-	rg_unlock(svcName, lockp);
+	rg_unlock(&lockp);
 
 	ret = group_op(svcName, RG_START);
 	ret = !!ret; /* Either it worked or it didn't.  Ignore all the
@@ -693,66 +777,167 @@ svc_start(char *svcName, int req)
 	if (rg_lock(svcName, &lockp) < 0) {
 		clulog(LOG_ERR, "#74: Unable to obtain cluster lock: %s\n",
 		       strerror(errno));
-		return FAIL;
+		return RG_EFAIL;
 	}
 
 	svcStatus.rs_state = RG_STATE_STARTED;
 	if (set_rg_state(svcName, &svcStatus) != 0) {
 		clulog(LOG_ERR,
 		       "#75: Failed changing service status\n");
-		rg_unlock(svcName, lockp);
-		return FAIL;
+		rg_unlock(&lockp);
+		return RG_EFAIL;
 	}
-	rg_unlock(svcName, lockp);
+	rg_unlock(&lockp);
        
-	if (ret == 0)
+	if (ret == 0) {
 		clulog(LOG_NOTICE,
 		       "Service %s started\n",
 		       svcName);
-	else
+
+		broadcast_event(svcName, RG_STATE_STARTED);
+	} else {
 		clulog(LOG_WARNING,
 		       "#68: Failed to start %s; return value: %d\n",
 		       svcName, ret);
+	}
 
 	return ret;
 }
 
 
+/**
+ * Migrate a service to another node.
+ */
+int
+svc_migrate(char *svcName, int target)
+{
+	struct dlm_lksb lockp;
+	rg_state_t svcStatus;
+	int ret;
+
+	if (!group_migratory(svcName))
+		return RG_EINVAL;
+
+	if (rg_lock(svcName, &lockp) < 0) {
+		clulog(LOG_ERR, "#45: Unable to obtain cluster lock: %s\n",
+		       strerror(errno));
+		return RG_EFAIL;
+	}
+
+	if (get_rg_state(svcName, &svcStatus) != 0) {
+		rg_unlock(&lockp);
+		clulog(LOG_ERR, "#46: Failed getting status for RG %s\n",
+		       svcName);
+		return RG_EFAIL;
+	}
+
+	if (svcStatus.rs_owner != my_id()) {
+		rg_unlock(&lockp);
+		return RG_EFORWARD;
+	}
+
+	switch(svcStatus.rs_state) {
+	case RG_STATE_STARTED:
+		break;
+	case RG_STATE_STARTING:
+	case RG_STATE_STOPPING:
+	case RG_STATE_RECOVER:
+	case RG_STATE_MIGRATE:
+	case RG_STATE_ERROR:
+		rg_unlock(&lockp);
+		return RG_EAGAIN;
+	default:
+		rg_unlock(&lockp);
+		return RG_EFAIL;
+	}
+
+	/* LOCK HELD */
+	svcStatus.rs_owner = target;
+	svcStatus.rs_last_owner = my_id();
+	svcStatus.rs_state = RG_STATE_MIGRATE;
+	svcStatus.rs_transition = (uint64_t)time(NULL);
+
+	if (set_rg_state(svcName, &svcStatus) != 0) {
+		clulog(LOG_ERR,
+		       "#75: Failed changing service status\n");
+		rg_unlock(&lockp);
+		return RG_EFAIL;
+	}
+	rg_unlock(&lockp);
+       
+	ret = group_migrate(svcName, target);
+	return ret;
+}
+
+
 /**
  * Check status of a cluster service 
  *
  * @param svcName	Service name to check.
- * @return		RG_EFORWARD, FAIL, 0
+ * @return		RG_EFORWARD, RG_EFAIL, 0
  */
 int
 svc_status(char *svcName)
 {
-	void *lockp = NULL;
+	struct dlm_lksb lockp;
 	rg_state_t svcStatus;
+	int ret;
 
 	if (rg_lock(svcName, &lockp) < 0) {
 		clulog(LOG_ERR, "#48: Unable to obtain cluster lock: %s\n",
 		       strerror(errno));
-		return FAIL;
+		return RG_EFAIL;
 	}
 
 	if (get_rg_state(svcName, &svcStatus) != 0) {
-		rg_unlock(svcName, lockp);
+		rg_unlock(&lockp);
 		clulog(LOG_ERR, "#49: Failed getting status for RG %s\n",
 		       svcName);
-		return FAIL;
+		return RG_EFAIL;
 	}
-	rg_unlock(svcName, lockp);
+	rg_unlock(&lockp);
 
 	if (svcStatus.rs_owner != my_id())
 		/* Don't check status for anything not owned */
-		return SUCCESS;
+		return 0;
 
-	if (svcStatus.rs_state != RG_STATE_STARTED)
+	if (svcStatus.rs_state != RG_STATE_STARTED &&
+	    svcStatus.rs_state != RG_STATE_MIGRATE)
 		/* Not-running RGs should not be checked either. */
-		return SUCCESS;
+		return 0;
 
-	return group_op(svcName, RG_STATUS);
+	ret = group_op(svcName, RG_STATUS);
+
+	/* For running services, just check the return code */
+	if (svcStatus.rs_state == RG_STATE_STARTED)
+		return ret;
+
+	/* For service(s) migrating to the local node, ignore invalid
+	   return codes.
+	   XXX Should put a timeout on migrating services */
+	if (ret < 0)
+		return 0;
+
+	/* If the check succeeds (returns 0), then flip the state back to
+	   'started' - we now own the service */
+	if (rg_lock(svcName, &lockp) < 0) {
+		clulog(LOG_ERR, "#45: Unable to obtain cluster lock: %s\n",
+			strerror(errno));
+		return RG_EFAIL;
+	}
+
+	svcStatus.rs_state = RG_STATE_STARTED;
+	if (set_rg_state(svcName, &svcStatus) != 0) {
+		rg_unlock(&lockp);
+		clulog(LOG_ERR, "#46: Failed getting status for RG %s\n",
+		       svcName);
+		return RG_EFAIL;
+	}
+	rg_unlock(&lockp);
+
+	clulog(LOG_NOTICE, "%s is now running locally\n", svcName);
+
+	return 0;
 }
 
 
@@ -767,7 +952,7 @@ svc_status(char *svcName)
 static int
 _svc_stop(char *svcName, int req, int recover, uint32_t newstate)
 {
-	void *lockp = NULL;
+	struct dlm_lksb lockp;
 	rg_state_t svcStatus;
 	int ret;
 	int old_state;
@@ -778,33 +963,33 @@ _svc_stop(char *svcName, int req, int recover, uint32_t newstate)
 		return group_op(svcName, RG_STOP);
 	}
 
-	if (rg_lock(svcName, &lockp) == FAIL) {
+	if (rg_lock(svcName, &lockp) == RG_EFAIL) {
 		clulog(LOG_ERR, "#50: Unable to obtain cluster lock: %s\n",
 		       strerror(errno));
-		return FAIL;
+		return RG_EFAIL;
 	}
 
 	if (get_rg_state(svcName, &svcStatus) != 0) {
-		rg_unlock(svcName, lockp);
+		rg_unlock(&lockp);
 		clulog(LOG_ERR, "#51: Failed getting status for RG %s\n",
 		       svcName);
-		return FAIL;
+		return RG_EFAIL;
 	}
 
 	switch (svc_advise_stop(&svcStatus, svcName, req)) {
 	case 0:
-		rg_unlock(svcName, lockp);
+		rg_unlock(&lockp);
 		clulog(LOG_DEBUG, "Unable to stop RG %s in %s state\n",
 		       svcName, rg_state_str(svcStatus.rs_state));
-		return FAIL;
+		return RG_EFAIL;
 	case 2:
-		rg_unlock(svcName, lockp);
-		return SUCCESS;
+		rg_unlock(&lockp);
+		return RG_ESUCCESS;
 	case 3:
-		rg_unlock(svcName, lockp);
+		rg_unlock(&lockp);
 		return RG_EFORWARD;
 	case 4:
-		rg_unlock(svcName, lockp);
+		rg_unlock(&lockp);
 		return RG_EAGAIN;
 	default:
 		break;
@@ -823,11 +1008,11 @@ _svc_stop(char *svcName, int req, int recover, uint32_t newstate)
 	//printf("rg state = %s\n", rg_state_str(svcStatus.rs_state));
 
 	if (set_rg_state(svcName, &svcStatus) != 0) {
-		rg_unlock(svcName, lockp);
+		rg_unlock(&lockp);
 		clulog(LOG_ERR, "#52: Failed changing RG status\n");
-		return FAIL;
+		return RG_EFAIL;
 	}
-	rg_unlock(svcName, lockp);
+	rg_unlock(&lockp);
 
 	ret = group_op(svcName, RG_STOP);
 
@@ -849,38 +1034,40 @@ static int
 _svc_stop_finish(char *svcName, int failed, uint32_t newstate)
 {
 	rg_state_t svcStatus;
-	void *lockp;
+	struct dlm_lksb lockp;
 
-	if (rg_lock(svcName, &lockp) == FAIL) {
+	if (rg_lock(svcName, &lockp) == RG_EFAIL) {
 		clulog(LOG_ERR, "#53: Unable to obtain cluster lock: %s\n",
 		       strerror(errno));
-		return FAIL;
+		return RG_EFAIL;
 	}
 
 	if (get_rg_state(svcName, &svcStatus) != 0) {
-		rg_unlock(svcName, lockp);
+		rg_unlock(&lockp);
 		clulog(LOG_ERR, "#54: Failed getting status for RG %s\n",
 		       svcName);
-		return FAIL;
+		return RG_EFAIL;
 	}
 
 	if ((svcStatus.rs_state != RG_STATE_STOPPING) &&
 	     (svcStatus.rs_state != RG_STATE_ERROR)) {
-		rg_unlock(svcName, lockp);
+		rg_unlock(&lockp);
 		return 0;
 	}
 
 	svcStatus.rs_last_owner = svcStatus.rs_owner;
-	svcStatus.rs_owner = NODE_ID_NONE;
+	svcStatus.rs_owner = 0;
 
 	if (failed) {
 		clulog(LOG_CRIT, "#12: RG %s failed to stop; intervention "
 		       "required\n", svcName);
-		svcStatus.rs_state = RG_STATE_FAILED;
-	} else if (svcStatus.rs_state == RG_STATE_ERROR)
+		newstate = RG_STATE_FAILED;
+	} else if (svcStatus.rs_state == RG_STATE_ERROR) {
 		svcStatus.rs_state = RG_STATE_RECOVER;
-	else
-		svcStatus.rs_state = newstate;
+		newstate = RG_STATE_RECOVER;
+	}
+
+	svcStatus.rs_state = newstate;
 
 	clulog(LOG_NOTICE, "Service %s is %s\n", svcName,
 	       rg_state_str(svcStatus.rs_state));
@@ -888,11 +1075,13 @@ _svc_stop_finish(char *svcName, int failed, uint32_t newstate)
 
 	svcStatus.rs_transition = (uint64_t)time(NULL);
 	if (set_rg_state(svcName, &svcStatus) != 0) {
-		rg_unlock(svcName, lockp);
+		rg_unlock(&lockp);
 		clulog(LOG_ERR, "#55: Failed changing RG status\n");
-		return FAIL;
+		return RG_EFAIL;
 	}
-	rg_unlock(svcName, lockp);
+	rg_unlock(&lockp);
+
+	broadcast_event(svcName, newstate);
 
 	return 0;
 }
@@ -930,48 +1119,50 @@ svc_stop(char *svcName, int req)
 int
 svc_fail(char *svcName)
 {
-	void *lockp = NULL;
+	struct dlm_lksb lockp;
 	rg_state_t svcStatus;
 
-	if (rg_lock(svcName, &lockp) == FAIL) {
+	if (rg_lock(svcName, &lockp) == RG_EFAIL) {
 		clulog(LOG_ERR, "#55: Unable to obtain cluster lock: %s\n",
 		       strerror(errno));
-		return FAIL;
+		return RG_EFAIL;
 	}
 
 	clulog(LOG_DEBUG, "Handling failure request for RG %s\n", svcName);
 
 	if (get_rg_state(svcName, &svcStatus) != 0) {
-		rg_unlock(svcName, lockp);
+		rg_unlock(&lockp);
 		clulog(LOG_ERR, "#56: Failed getting status for RG %s\n",
 		       svcName);
-		return FAIL;
+		return RG_EFAIL;
 	}
 
 	if ((svcStatus.rs_state == RG_STATE_STARTED) &&
 	    (svcStatus.rs_owner != my_id())) {
-		rg_unlock(svcName, lockp);
+		rg_unlock(&lockp);
 		clulog(LOG_DEBUG, "Unable to disable RG %s in %s state\n",
 		       svcName, rg_state_str(svcStatus.rs_state));
-		return FAIL;
+		return RG_EFAIL;
 	}
 
 	/*
 	 * Leave a bread crumb so we can debug the problem with the service!
 	 */
-	if (svcStatus.rs_owner != NODE_ID_NONE) {
+	if (svcStatus.rs_owner != 0) {
 		svcStatus.rs_last_owner = svcStatus.rs_owner;
-		svcStatus.rs_owner = NODE_ID_NONE;
+		svcStatus.rs_owner = 0;
 	}
 	svcStatus.rs_state = RG_STATE_FAILED;
 	svcStatus.rs_transition = (uint64_t)time(NULL);
 	svcStatus.rs_restarts = 0;
 	if (set_rg_state(svcName, &svcStatus) != 0) {
-		rg_unlock(svcName, lockp);
+		rg_unlock(&lockp);
 		clulog(LOG_ERR, "#57: Failed changing RG status\n");
-		return FAIL;
+		return RG_EFAIL;
 	}
-	rg_unlock(svcName, lockp);
+	rg_unlock(&lockp);
+
+	broadcast_event(svcName, RG_STATE_FAILED);
 
 	return 0;
 }
@@ -981,11 +1172,12 @@ svc_fail(char *svcName)
  * Send a message to the target node to start the service.
  */
 static int
-relocate_service(char *svcName, int request, uint64_t target)
+relocate_service(char *svcName, int request, uint32_t target)
 {
 	SmMessageSt msg_relo;
-	int fd_relo, msg_ret;
+	int msg_ret;
 	cluster_member_list_t *ml;
+	msgctx_t ctx;
 
 	/* Build the message header */
 	msg_relo.sm_hdr.gh_magic = GENERIC_HDR_MAGIC;
@@ -998,7 +1190,7 @@ relocate_service(char *svcName, int request, uint64_t target)
 
 	/* Open a connection to the other node */
 
-	if ((fd_relo = msg_open(target, RG_PORT, RG_PURPOSE, 2)) < 0) {
+	if (msg_open(MSG_CLUSTER, target, RG_PORT, &ctx, 2)< 0) {
 		clulog(LOG_ERR,
 		       "#58: Failed opening connection to member #%d\n",
 		       target);
@@ -1009,12 +1201,12 @@ relocate_service(char *svcName, int request, uint64_t target)
 	swab_SmMessageSt(&msg_relo);
 
 	/* Send relocate message to the other node */
-	if (msg_send(fd_relo, &msg_relo, sizeof (SmMessageSt)) !=
+	if (msg_send(&ctx, &msg_relo, sizeof (SmMessageSt)) < 
 	    sizeof (SmMessageSt)) {
 		clulog(LOG_ERR,
 		       "#59: Error sending relocate request to member #%d\n",
 		       target);
-		msg_close(fd_relo);
+		msg_close(&ctx);
 		return -1;
 	}
 
@@ -1022,7 +1214,7 @@ relocate_service(char *svcName, int request, uint64_t target)
 
 	/* Check the response */
 	do {
-		msg_ret = msg_receive_timeout(fd_relo, &msg_relo,
+		msg_ret = msg_receive(&ctx, &msg_relo,
 					      sizeof (SmMessageSt), 10);
 		if ((msg_ret == -1 && errno != ETIMEDOUT) ||
 		    (msg_ret >= 0)) {
@@ -1034,21 +1226,21 @@ relocate_service(char *svcName, int request, uint64_t target)
 		if (rg_locked()) {
 			clulog(LOG_WARNING,
 			       "#XX: Cancelling relocation: Shutting down\n");
-			msg_close(fd_relo);
-			return NO;
+			msg_close(&ctx);
+			return RG_NO;
 		}
 
 		/* Check for node transition in the middle of a relocate */
 		ml = member_list();
 		if (memb_online(ml, target)) {
-			cml_free(ml);
+			free_member_list(ml);
 			continue;
 		}
 		clulog(LOG_WARNING,
 		       "#XX: Cancelling relocation: Target node down\n");
-		cml_free(ml);
-		msg_close(fd_relo);
-		return FAIL;
+		free_member_list(ml);
+		msg_close(&ctx);
+		return RG_EFAIL;
 	} while (1);
 
 	if (msg_ret != sizeof (SmMessageSt)) {
@@ -1058,12 +1250,12 @@ relocate_service(char *svcName, int request, uint64_t target)
 		 */
 		clulog(LOG_ERR, "#60: Mangled reply from member #%d during RG "
 		       "relocate\n", target);
-		msg_close(fd_relo);
+		msg_close(&ctx);
 		return 0;	/* XXX really UNKNOWN */
 	}
 
 	/* Got a valid response from other node. */
-	msg_close(fd_relo);
+	msg_close(&ctx);
 
 	/* Decode */
 	swab_SmMessageSt(&msg_relo);
@@ -1092,11 +1284,11 @@ relocate_service(char *svcName, int request, uint64_t target)
  * @param new_owner	Member who actually ends up owning the service.
  */
 int
-handle_relocate_req(char *svcName, int request, uint64_t preferred_target,
-		    uint64_t *new_owner)
+handle_relocate_req(char *svcName, int request, int preferred_target,
+		    int *new_owner)
 {
 	cluster_member_list_t *allowed_nodes, *backup = NULL;
-	uint64_t target = preferred_target, me = my_id();
+	int target = preferred_target, me = my_id();
 	int ret, x;
 	
 	/*
@@ -1104,15 +1296,15 @@ handle_relocate_req(char *svcName, int request, uint64_t preferred_target,
 	 */
 	if (request != RG_START_RECOVER) {
 		ret = _svc_stop(svcName, request, 0, RG_STATE_STOPPED);
-		if (ret == FAIL) {
+		if (ret == RG_EFAIL) {
 			svc_fail(svcName);
-			return FAIL;
+			return RG_EFAIL;
 		}
 		if (ret == RG_EFORWARD)
 			return RG_EFORWARD;
 	}
 
-	if (preferred_target != NODE_ID_NONE) {
+	if (preferred_target >= 0) {
 
 		allowed_nodes = member_list();
 		/*
@@ -1121,14 +1313,14 @@ handle_relocate_req(char *svcName, int request, uint64_t preferred_target,
 	 	   other nodes.
 		 */
 		//count_resource_groups(allowed_nodes);
-		backup = cml_dup(allowed_nodes);
+		backup = member_list_dup(allowed_nodes);
 
 		for (x = 0; x < allowed_nodes->cml_count; x++) {
-			if (allowed_nodes->cml_members[x].cm_id == me ||
-		    	    allowed_nodes->cml_members[x].cm_id ==
+			if (allowed_nodes->cml_members[x].cn_nodeid == me ||
+		    	    allowed_nodes->cml_members[x].cn_nodeid ==
 			    		preferred_target)
 				continue;
-			allowed_nodes->cml_members[x].cm_state = STATE_DOWN;
+			allowed_nodes->cml_members[x].cn_member = 0;
 		}
 
 		/*
@@ -1139,7 +1331,7 @@ handle_relocate_req(char *svcName, int request, uint64_t preferred_target,
 		 */
 		target = best_target_node(allowed_nodes, me, svcName, 1);
 
-		cml_free(allowed_nodes);
+		free_member_list(allowed_nodes);
 
 		/*
 		 * I am the ONLY one capable of running this service,
@@ -1184,7 +1376,7 @@ handle_relocate_req(char *svcName, int request, uint64_t preferred_target,
 		//count_resource_groups(allowed_nodes);
 	}
 
-	if (preferred_target != NODE_ID_NONE)
+	if (preferred_target >= 0)
 		memb_mark_down(allowed_nodes, preferred_target);
 	memb_mark_down(allowed_nodes, me);
 
@@ -1199,21 +1391,20 @@ handle_relocate_req(char *svcName, int request, uint64_t preferred_target,
 			continue;
 		case RG_EABORT:
 			svc_report_failure(svcName);
-			cml_free(allowed_nodes);
-			return FAIL;
-		case NO:
+			free_member_list(allowed_nodes);
+			return RG_EFAIL;
+		case RG_NO:
 			/* state uncertain */
-			cml_free(allowed_nodes);
+			free_member_list(allowed_nodes);
 			clulog(LOG_DEBUG, "State Uncertain: svc:%s "
-			       "nid:%08x%08x req:%d\n", svcName,
-			       (uint32_t)(target>>32)&0xffffffff,
-			       (uint32_t)(target&0xffffffff), request);
+			       "nid:%08x req:%d\n", svcName,
+			       target, request);
 			return 0;
 		case 0:
 			*new_owner = target;
 			clulog(LOG_NOTICE, "Service %s is now running "
 			       "on member %d\n", svcName, (int)target);
-			cml_free(allowed_nodes);
+			free_member_list(allowed_nodes);
 			return 0;
 		default:
 			clulog(LOG_ERR,
@@ -1221,14 +1412,14 @@ handle_relocate_req(char *svcName, int request, uint64_t preferred_target,
 			       " relocate operation!\n", target);
 		}
 	}
-	cml_free(allowed_nodes);
+	free_member_list(allowed_nodes);
 
 	/*
 	 * We got sent here from handle_start_req.
 	 * We're DONE.
 	 */
 	if (request == RG_START_RECOVER)
-		return FAIL;
+		return RG_EFAIL;
 
 	/*
 	 * All potential places for the service to start have been exhausted.
@@ -1241,7 +1432,7 @@ exhausted:
 		       svcName);
 		if (svc_start(svcName, RG_START_RECOVER) == 0) {
 			*new_owner = me;
-			return FAIL;
+			return RG_EFAIL;
 		}
 	}
 
@@ -1250,60 +1441,6 @@ exhausted:
 		svc_report_failure(svcName);
 	}
 
-	return FAIL;
-}
-
-
-int
-handle_fd_start_req(char *svcName, int request, uint64_t *new_owner)
-{
-	cluster_member_list_t *allowed_nodes, *backup = NULL;
-	uint64_t target, me = my_id();
-	int ret, x;
-	
-	allowed_nodes = member_list();
-
-	while (memb_count(allowed_nodes)) {
-		target = best_target_node(allowed_nodes, NODE_ID_NONE,
-					  svcName, 1);
-		if (target == me) {
-			ret = handle_start_remote_req(svcName, request);
-		} else {
-			ret = relocate_service(svcName, request, target);
-		}
-
-		switch (ret) {
-		case RG_ERUN:
-			return RG_ERUN;
-		case RG_EFAIL:
-			memb_mark_down(allowed_nodes, target);
-			continue;
-		case RG_EABORT:
-			svc_report_failure(svcName);
-			cml_free(allowed_nodes);
-			return FAIL;
-		case NO:
-			/* state uncertain */
-			cml_free(allowed_nodes);
-			clulog(LOG_DEBUG, "State Uncertain: svc:%s "
-			       "nid:%08x%08x req:%d\n", svcName,
-			       (uint32_t)(target>>32)&0xffffffff,
-			       (uint32_t)(target&0xffffffff), request);
-			return 0;
-		case 0:
-			*new_owner = target;
-			clulog(LOG_NOTICE, "Service %s is now running "
-			       "on member %d\n", svcName, (int)target);
-			cml_free(allowed_nodes);
-			return 0;
-		default:
-			clulog(LOG_ERR,
-			       "#61: Invalid reply from member %d during"
-			       " relocate operation!\n", target);
-		}
-	}
-	
-	cml_free(allowed_nodes);
 	return RG_EFAIL;
 }
 
@@ -1319,7 +1456,7 @@ handle_fd_start_req(char *svcName, int request, uint64_t *new_owner)
  *			0 - The service is running.
  */
 int
-handle_start_req(char *svcName, int req, uint64_t *new_owner)
+handle_start_req(char *svcName, int req, int *new_owner)
 {
 	int ret, tolerance = FOD_BEST;
 	cluster_member_list_t *membership = member_list();
@@ -1335,10 +1472,15 @@ handle_start_req(char *svcName, int req, uint64_t *new_owner)
 	    req != RG_START_RECOVER &&
 	    (node_should_start_safe(my_id(), membership, svcName) <
 	     tolerance)) {
-		cml_free(membership);
-		return FAIL;
+		free_member_list(membership);
+		return RG_EFAIL;
 	}
-	cml_free(membership);
+	free_member_list(membership);
+
+	/* Check for dependency.  We cannot start unless our
+	   dependency is met */
+	if (check_depend_safe(svcName) == 0)
+		return RG_EDEPEND;
 	
 	/*
 	 * This is a 'root' start request.  We need to clear out our failure
@@ -1355,14 +1497,14 @@ handle_start_req(char *svcName, int req, uint64_t *new_owner)
 	/*
 	 * If we succeeded, then we're done.
 	 */
-	if (ret == SUCCESS) {
+	if (ret == RG_ESUCCESS) {
 		*new_owner = my_id();
-		return SUCCESS;
+		return RG_ESUCCESS;
 	}
 
 	/* Already running? */
-	if (ret == NO) {
-		return SUCCESS;
+	if (ret == RG_NO) {
+		return RG_ESUCCESS;
 	}
 	
 	/* 
@@ -1394,7 +1536,7 @@ handle_start_req(char *svcName, int req, uint64_t *new_owner)
 
 	/* If we leave the service stopped, instead of disabled, someone
 	   will try to start it after the next node transition */
-	if (ret == FAIL) {
+	if (ret == RG_EFAIL) {
 		if (svc_stop(svcName, RG_STOP) != 0) {
 			svc_fail(svcName);
 			svc_report_failure(svcName);
@@ -1419,16 +1561,16 @@ handle_start_remote_req(char *svcName, int req)
 {
 	int tolerance = FOD_BEST;
 	int x;
-	uint64_t me = my_id();
+	uint32_t me = my_id();
 	cluster_member_list_t *membership = member_list();
 
 	/* XXX ok, so we need to say "should I start this if I was the
 	   only cluster member online */
 	for (x = 0; x < membership->cml_count; x++) {
-		if (membership->cml_members[x].cm_id == me)
+		if (membership->cml_members[x].cn_nodeid == me)
 			continue;
 
-		membership->cml_members[x].cm_state = STATE_DOWN;
+		membership->cml_members[x].cn_member = 0;
 	}
 
 	if (req == RG_ENABLE)
@@ -1438,16 +1580,13 @@ handle_start_remote_req(char *svcName, int req)
 	 * See if we agree with our ability to start the given service.
 	 */
 	if (node_should_start_safe(me, membership, svcName) < tolerance){
-		cml_free(membership);
-		return FAIL;
+		free_member_list(membership);
+		return RG_EFAIL;
 	}
-	cml_free(membership);
+	free_member_list(membership);
 
-	x = svc_start(svcName, req);
-	if (x == 0)
+	if (svc_start(svcName, req) == 0)
 		return 0;
-	if (x == RG_ERUN)
-		return RG_ERUN;
 
 	if (svc_stop(svcName, RG_STOP_RECOVER) == 0)
 		return RG_EFAIL;
@@ -1461,7 +1600,7 @@ handle_start_remote_req(char *svcName, int req)
   handle_recover_req
  */
 int
-handle_recover_req(char *svcName, uint64_t *new_owner)
+handle_recover_req(char *svcName, int *new_owner)
 {
 	char policy[20];
 
