@@ -21,6 +21,7 @@
 #include <gettid.h>
 #include <rg_queue.h>
 #include <assert.h>
+#include <message.h>
 
 /**
  * Resource thread list entry.
@@ -47,7 +48,7 @@ static pthread_mutex_t reslist_mutex = PTHREAD_MUTEX_INITIALIZER;
 static resthread_t *find_resthread_byname(const char *resgroupname);
 static int spawn_if_needed(const char *resgroupname);
 int rt_enqueue_request(const char *resgroupname, int request,
-		       int response_fd, int max, uint64_t target,
+		       msgctx_t *response_ctx, int max, uint32_t target,
 		       int arg0, int arg1);
 
 
@@ -144,8 +145,8 @@ purge_all(request_t **list)
 
 		list_remove(list, curr);
 		dprintf("Removed request %d\n", curr->rr_request);
-		if (curr->rr_resp_fd != -1) {
-			send_response(RG_EABORT, NODE_ID_NONE, curr);
+		if (curr->rr_resp_ctx) {
+			send_response(RG_EABORT, 0, curr);
 		}
 		rq_free(curr);
 	}
@@ -158,7 +159,7 @@ resgroup_thread_main(void *arg)
 	pthread_mutex_t my_queue_mutex;
 	pthread_cond_t my_queue_cond;
 	request_t *my_queue = NULL;
-	uint64_t newowner = NODE_ID_NONE;
+	int newowner = 0;
 	char myname[256];
 	resthread_t *myself;
 	request_t *req;
@@ -235,17 +236,16 @@ resgroup_thread_main(void *arg)
 							req->rr_request);
 			break;
 
-		case RG_START:
 		case RG_ENABLE:
-			if (req->rr_arg0) {
-				error = handle_fd_start_req(myname,
-						            req->rr_request,
-						            &newowner);
-			} else {
-				error = handle_start_req(myname,
-							 req->rr_request,
-							 &newowner);
+			if (req->rr_target != 0 &&
+			    req->rr_target != my_id()) {
+				error = RG_EFORWARD;
+				ret = RG_NONE;
+				break;
 			}
+		case RG_START:
+			error = handle_start_req(myname, req->rr_request,
+						 &newowner);
 			break;
 
 		case RG_RELOCATE:
@@ -258,6 +258,26 @@ resgroup_thread_main(void *arg)
 				ret = RG_NONE;
 			break;
 
+		case RG_MIGRATE:
+			error = svc_migrate(myname, req->rr_target);
+
+			if (error == 0) {
+				ret = RG_SUCCESS;
+
+				pthread_mutex_lock(&my_queue_mutex);
+				purge_status_checks(&my_queue);
+				pthread_mutex_unlock(&my_queue_mutex);
+			} else if (error == RG_EFORWARD) {
+				ret = RG_NONE;
+				break;
+			} else {
+				/*
+				 * Bad news. 
+				 */
+				ret = RG_FAIL;
+			}
+			break;
+
 		case RG_INIT:
 			/* Stop without changing shared state of it */
 			error = group_op(myname, RG_STOP);
@@ -418,7 +438,7 @@ resgroup_thread_main(void *arg)
 		}
 
 		if (ret != RG_NONE && rg_initialized() &&
-		    (req->rr_resp_fd >= 0)) {
+		    (req->rr_resp_ctx)) {
 			send_response(error, newowner, req);
 		}
 		
@@ -546,7 +566,7 @@ find_resthread_byname(const char *resgroupname)
  *
  * @param resgroupname		Service name to perform operations on
  * @param request		Request to perform
- * @param response_fd		Send response to this file descriptor when
+ * @param response_ctx		Send response to this file descriptor when
  *				this request completes.
  * @param max			Don't insert this request if there already
  * 				are this many requests of this type in the
@@ -558,8 +578,9 @@ find_resthread_byname(const char *resgroupname)
  * @see rq_queue_request
  */
 int
-rt_enqueue_request(const char *resgroupname, int request, int response_fd,
-   		   int max, uint64_t target, int arg0, int arg1)
+rt_enqueue_request(const char *resgroupname, int request, 
+		   msgctx_t *response_ctx,
+   		   int max, uint32_t target, int arg0, int arg1)
 {
 	request_t *curr;
 	int count = 0, ret;
@@ -616,8 +637,8 @@ rt_enqueue_request(const char *resgroupname, int request, int response_fd,
 		case RG_START_RECOVER:
 		case RG_START:
 		case RG_ENABLE:
-			send_ret(response_fd, resgroup->rt_name, RG_EDEADLCK,
-				 request, NODE_ID_NONE);
+			send_ret(response_ctx, resgroup->rt_name, RG_EDEADLCK,
+				 request);
 			break;
 		}
 		fprintf(stderr, "Failed to queue request: Would block\n");
@@ -628,7 +649,7 @@ rt_enqueue_request(const char *resgroupname, int request, int response_fd,
 	}
 
 	ret = rq_queue_request(resgroup->rt_queue, resgroup->rt_name,
-			       request, 0, 0, response_fd, 0, target,
+			       request, 0, 0, response_ctx, 0, target,
 			       arg0, arg1);
 	pthread_cond_broadcast(resgroup->rt_queue_cond);
 	pthread_mutex_unlock(resgroup->rt_queue_mutex);
