@@ -95,18 +95,6 @@ meta_data()
             <content type="string"/>
         </parameter>
 
-        <parameter name="nfslock" inherit="nfsexport%nfslock">
-            <longdesc lang="en">
-	    	This tells us whether the service in question has the
-		NFS lock workarounds enabled.  If so, we always unexport
-		* rather than the specified client.
-            </longdesc>
-            <shortdesc lang="en">
-	    	NFS Lock workaround flag
-            </shortdesc>
-            <content type="boolean"/>
-        </parameter>
-
         <parameter name="options">
             <longdesc lang="en">Defines a list of options for this
                 particular client.  See 'man 5 exports' for a list
@@ -307,14 +295,6 @@ start)
 stop)
 	verify_all || exit $OCF_ERR_ARGS
 
-	if [ "$OCF_RESKEY_nfslock" = "1" ]; then
-		#
-		# If the NFS lock workarounds were enabled, unexport from
-		# the world
-		#
-		export OCF_RESKEY_target="*"
-	fi
-
 	ocf_log info "Removing export: ${OCF_RESKEY_target}:${OCF_RESKEY_path}"
 	exportfs -u "${OCF_RESKEY_target}:${OCF_RESKEY_path}"
 	rv=$?
